// -*- mode: C++ -*-

// AUTOGENERATED BY glean_parser.  DO NOT EDIT.

/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */

#include "mozilla/glean/bindings/GleanJSMetricsLookup.h"

#include "mozilla/PerfectHash.h"
#include "mozilla/Maybe.h"
#include "mozilla/glean/bindings/MetricTypes.h"
#include "mozilla/glean/fog_ffi_generated.h"
#include "nsString.h"

#define GLEAN_INDEX_BITS (32)
#define GLEAN_TYPE_BITS (5)
#define GLEAN_ID_BITS (27)
#define GLEAN_TYPE_ID(id) ((id) >> GLEAN_ID_BITS)
#define GLEAN_METRIC_ID(id) ((id) & ((1ULL << GLEAN_ID_BITS) - 1))
#define GLEAN_OFFSET(entry) (entry & ((1ULL << GLEAN_INDEX_BITS) - 1))

namespace mozilla::glean {

// The category lookup table's entry type
using category_entry_t = uint32_t;
// The metric lookup table's entry type
// This is a bitpacked type with 32 bits available to index into
// the string table, 5 bits available to signify the metric type,
// and the remaining 27 bits devoted to 2 "signal"
// bits to signify important characteristics (metric's a labeled metric's
// submetric, metric's been registered at runtime) and 25 bits
// for built-in metric ids.
// Gives room for 33554432 of each combination of
// characteristics (which hopefully will prove to be enough).
using metric_entry_t = uint64_t;

static_assert(GLEAN_INDEX_BITS + GLEAN_TYPE_BITS + GLEAN_ID_BITS == sizeof(metric_entry_t) * 8, "Index, Type, and ID bits need to fit into a metric_entry_t");
static_assert(GLEAN_TYPE_BITS + GLEAN_ID_BITS <= sizeof(uint32_t) * 8, "Metric Types and IDs need to fit into at most 32 bits");
static_assert(64 < UINT32_MAX, "Too many metric categories generated.");
static_assert(383 < 33554432, "Too many metrics generated. Need room for 2 signal bits.");
static_assert(20 < 32, "Too many different metric types.");

already_AddRefed<nsISupports> NewMetricFromId(uint32_t id) {
  uint32_t typeId = GLEAN_TYPE_ID(id);
  uint32_t metricId = GLEAN_METRIC_ID(id);

  switch (typeId) {
    case 1: /* string_list */
    {
      return MakeAndAddRef<GleanStringList>(metricId);
    }
    case 2: /* datetime */
    {
      return MakeAndAddRef<GleanDatetime>(metricId);
    }
    case 3: /* uuid */
    {
      return MakeAndAddRef<GleanUuid>(metricId);
    }
    case 4: /* boolean */
    {
      return MakeAndAddRef<GleanBoolean>(metricId);
    }
    case 5: /* string */
    {
      return MakeAndAddRef<GleanString>(metricId);
    }
    case 6: /* quantity */
    {
      return MakeAndAddRef<GleanQuantity>(metricId);
    }
    case 7: /* event */
    {
      return MakeAndAddRef<GleanEvent>(metricId);
    }
    case 8: /* text */
    {
      return MakeAndAddRef<GleanText>(metricId);
    }
    case 9: /* counter */
    {
      return MakeAndAddRef<GleanCounter>(metricId);
    }
    case 10: /* labeled_counter */
    {
      return MakeAndAddRef<GleanLabeled>(metricId, 10);
    }
    case 11: /* url */
    {
      return MakeAndAddRef<GleanUrl>(metricId);
    }
    case 12: /* timing_distribution */
    {
      return MakeAndAddRef<GleanTimingDistribution>(metricId);
    }
    case 13: /* denominator */
    {
      return MakeAndAddRef<GleanDenominator>(metricId);
    }
    case 14: /* numerator */
    {
      return MakeAndAddRef<GleanNumerator>(metricId);
    }
    case 15: /* custom_distribution */
    {
      return MakeAndAddRef<GleanCustomDistribution>(metricId);
    }
    case 16: /* labeled_boolean */
    {
      return MakeAndAddRef<GleanLabeled>(metricId, 16);
    }
    case 17: /* timespan */
    {
      return MakeAndAddRef<GleanTimespan>(metricId);
    }
    case 18: /* memory_distribution */
    {
      return MakeAndAddRef<GleanMemoryDistribution>(metricId);
    }
    case 19: /* labeled_string */
    {
      return MakeAndAddRef<GleanLabeled>(metricId, 19);
    }
    case 20: /* rate */
    {
      return MakeAndAddRef<GleanRate>(metricId);
    }
    default:
    MOZ_ASSERT_UNREACHABLE("Invalid type ID reached when trying to instantiate a new metric");
    return nullptr;
  }
}

/**
 * Create a submetric instance for a labeled metric of the provided type and id for the given label.
 * Assigns or retrieves an id for the submetric from the SDK.
 *
 * @param aParentTypeId - The type of the parent labeled metric identified as a number generated during codegen.
 *                        Only used to identify which X of LabeledX you are so that X can be created here.
 * @param aParentMetricId - The metric id for the parent labeled metric.
 * @param aLabel - The label for the submetric. Might not adhere to the SDK label format.
 * @param aSubmetricId - an outparam which is assigned the submetric's SDK-generated submetric id.
 *                       Used only by GIFFT.
 */
already_AddRefed<nsISupports> NewSubMetricFromIds(uint32_t aParentTypeId, uint32_t aParentMetricId, const nsACString& aLabel, uint32_t* aSubmetricId) {
  switch (aParentTypeId) {
    case 10: { /* labeled_counter */
      auto id = impl::fog_labeled_counter_get(aParentMetricId, &aLabel);
      *aSubmetricId = id;
      return MakeAndAddRef<GleanCounter>(id);
    }
    case 16: { /* labeled_boolean */
      auto id = impl::fog_labeled_boolean_get(aParentMetricId, &aLabel);
      *aSubmetricId = id;
      return MakeAndAddRef<GleanBoolean>(id);
    }
    case 19: { /* labeled_string */
      auto id = impl::fog_labeled_string_get(aParentMetricId, &aLabel);
      *aSubmetricId = id;
      return MakeAndAddRef<GleanString>(id);
    }
    default: {
      MOZ_ASSERT_UNREACHABLE("Invalid type ID for submetric.");
      return nullptr;
    }
  }
}

static Maybe<uint32_t> category_result_check(const nsACString& aKey, category_entry_t entry);
static Maybe<uint32_t> metric_result_check(const nsACString& aKey, metric_entry_t entry);

#if defined(_MSC_VER) && !defined(__clang__)
const char gCategoryStringTable[] = {
#else
constexpr char gCategoryStringTable[] = {
#endif
  /*     0 - "backgroundUpdate" */ 'b', 'a', 'c', 'k', 'g', 'r', 'o', 'u', 'n', 'd', 'U', 'p', 'd', 'a', 't', 'e', '\0',
  /*    17 - "browserLaunchedToHandle" */ 'b', 'r', 'o', 'w', 's', 'e', 'r', 'L', 'a', 'u', 'n', 'c', 'h', 'e', 'd', 'T', 'o', 'H', 'a', 'n', 'd', 'l', 'e', '\0',
  /*    41 - "browserMigration" */ 'b', 'r', 'o', 'w', 's', 'e', 'r', 'M', 'i', 'g', 'r', 'a', 't', 'i', 'o', 'n', '\0',
  /*    58 - "messagingSystem" */ 'm', 'e', 's', 's', 'a', 'g', 'i', 'n', 'g', 'S', 'y', 's', 't', 'e', 'm', '\0',
  /*    74 - "messagingSystemAttribution" */ 'm', 'e', 's', 's', 'a', 'g', 'i', 'n', 'g', 'S', 'y', 's', 't', 'e', 'm', 'A', 't', 't', 'r', 'i', 'b', 'u', 't', 'i', 'o', 'n', '\0',
  /*   101 - "newtab" */ 'n', 'e', 'w', 't', 'a', 'b', '\0',
  /*   108 - "newtabSearch" */ 'n', 'e', 'w', 't', 'a', 'b', 'S', 'e', 'a', 'r', 'c', 'h', '\0',
  /*   121 - "pocket" */ 'p', 'o', 'c', 'k', 'e', 't', '\0',
  /*   128 - "topSites" */ 't', 'o', 'p', 'S', 'i', 't', 'e', 's', '\0',
  /*   137 - "topsites" */ 't', 'o', 'p', 's', 'i', 't', 'e', 's', '\0',
  /*   146 - "privacyUiFppClick" */ 'p', 'r', 'i', 'v', 'a', 'c', 'y', 'U', 'i', 'F', 'p', 'p', 'C', 'l', 'i', 'c', 'k', '\0',
  /*   164 - "newtabSearchAd" */ 'n', 'e', 'w', 't', 'a', 'b', 'S', 'e', 'a', 'r', 'c', 'h', 'A', 'd', '\0',
  /*   179 - "serp" */ 's', 'e', 'r', 'p', '\0',
  /*   184 - "shopping" */ 's', 'h', 'o', 'p', 'p', 'i', 'n', 'g', '\0',
  /*   193 - "shoppingSettings" */ 's', 'h', 'o', 'p', 'p', 'i', 'n', 'g', 'S', 'e', 't', 't', 'i', 'n', 'g', 's', '\0',
  /*   210 - "quickSuggest" */ 'q', 'u', 'i', 'c', 'k', 'S', 'u', 'g', 'g', 'e', 's', 't', '\0',
  /*   223 - "urlbar" */ 'u', 'r', 'l', 'b', 'a', 'r', '\0',
  /*   230 - "browserEngagement" */ 'b', 'r', 'o', 'w', 's', 'e', 'r', 'E', 'n', 'g', 'a', 'g', 'e', 'm', 'e', 'n', 't', '\0',
  /*   248 - "pingCentre" */ 'p', 'i', 'n', 'g', 'C', 'e', 'n', 't', 'r', 'e', '\0',
  /*   259 - "gmp" */ 'g', 'm', 'p', '\0',
  /*   263 - "rtcrtpsender" */ 'r', 't', 'c', 'r', 't', 'p', 's', 'e', 'n', 'd', 'e', 'r', '\0',
  /*   276 - "rtcrtpsenderSetparameters" */ 'r', 't', 'c', 'r', 't', 'p', 's', 'e', 'n', 'd', 'e', 'r', 'S', 'e', 't', 'p', 'a', 'r', 'a', 'm', 'e', 't', 'e', 'r', 's', '\0',
  /*   302 - "perf" */ 'p', 'e', 'r', 'f', '\0',
  /*   307 - "paint" */ 'p', 'a', 'i', 'n', 't', '\0',
  /*   313 - "wr" */ 'w', 'r', '\0',
  /*   316 - "dotprint" */ 'd', 'o', 't', 'p', 'r', 'i', 'n', 't', '\0',
  /*   325 - "networking" */ 'n', 'e', 't', 'w', 'o', 'r', 'k', 'i', 'n', 'g', '\0',
  /*   336 - "netwerk" */ 'n', 'e', 't', 'w', 'e', 'r', 'k', '\0',
  /*   344 - "network" */ 'n', 'e', 't', 'w', 'o', 'r', 'k', '\0',
  /*   352 - "backgroundTasksRmdirBase" */ 'b', 'a', 'c', 'k', 'g', 'r', 'o', 'u', 'n', 'd', 'T', 'a', 's', 'k', 's', 'R', 'm', 'd', 'i', 'r', 'B', 'a', 's', 'e', '\0',
  /*   377 - "backgroundTasksRmdirHttpCache" */ 'b', 'a', 'c', 'k', 'g', 'r', 'o', 'u', 'n', 'd', 'T', 'a', 's', 'k', 's', 'R', 'm', 'd', 'i', 'r', 'H', 't', 't', 'p', 'C', 'a', 'c', 'h', 'e', '\0',
  /*   407 - "backgroundTasksRmdirQuota" */ 'b', 'a', 'c', 'k', 'g', 'r', 'o', 'u', 'n', 'd', 'T', 'a', 's', 'k', 's', 'R', 'm', 'd', 'i', 'r', 'Q', 'u', 'o', 't', 'a', '\0',
  /*   433 - "cookieBanners" */ 'c', 'o', 'o', 'k', 'i', 'e', 'B', 'a', 'n', 'n', 'e', 'r', 's', '\0',
  /*   447 - "cookieBannersClick" */ 'c', 'o', 'o', 'k', 'i', 'e', 'B', 'a', 'n', 'n', 'e', 'r', 's', 'C', 'l', 'i', 'c', 'k', '\0',
  /*   466 - "crash" */ 'c', 'r', 'a', 's', 'h', '\0',
  /*   472 - "extensions" */ 'e', 'x', 't', 'e', 'n', 's', 'i', 'o', 'n', 's', '\0',
  /*   483 - "extensionsApisDnr" */ 'e', 'x', 't', 'e', 'n', 's', 'i', 'o', 'n', 's', 'A', 'p', 'i', 's', 'D', 'n', 'r', '\0',
  /*   501 - "extensionsQuarantinedDomains" */ 'e', 'x', 't', 'e', 'n', 's', 'i', 'o', 'n', 's', 'Q', 'u', 'a', 'r', 'a', 'n', 't', 'i', 'n', 'e', 'd', 'D', 'o', 'm', 'a', 'i', 'n', 's', '\0',
  /*   530 - "formautofillCreditcards" */ 'f', 'o', 'r', 'm', 'a', 'u', 't', 'o', 'f', 'i', 'l', 'l', 'C', 'r', 'e', 'd', 'i', 't', 'c', 'a', 'r', 'd', 's', '\0',
  /*   554 - "fog" */ 'f', 'o', 'g', '\0',
  /*   558 - "fogIpc" */ 'f', 'o', 'g', 'I', 'p', 'c', '\0',
  /*   565 - "fogValidation" */ 'f', 'o', 'g', 'V', 'a', 'l', 'i', 'd', 'a', 't', 'i', 'o', 'n', '\0',
  /*   579 - "testOnly" */ 't', 'e', 's', 't', 'O', 'n', 'l', 'y', '\0',
  /*   588 - "testOnlyIpc" */ 't', 'e', 's', 't', 'O', 'n', 'l', 'y', 'I', 'p', 'c', '\0',
  /*   600 - "testOnlyJog" */ 't', 'e', 's', 't', 'O', 'n', 'l', 'y', 'J', 'o', 'g', '\0',
  /*   612 - "nimbusEvents" */ 'n', 'i', 'm', 'b', 'u', 's', 'E', 'v', 'e', 'n', 't', 's', '\0',
  /*   625 - "pwmgr" */ 'p', 'w', 'm', 'g', 'r', '\0',
  /*   631 - "pdfjs" */ 'p', 'd', 'f', 'j', 's', '\0',
  /*   637 - "power" */ 'p', 'o', 'w', 'e', 'r', '\0',
  /*   643 - "powerBattery" */ 'p', 'o', 'w', 'e', 'r', 'B', 'a', 't', 't', 'e', 'r', 'y', '\0',
  /*   656 - "powerCpuMsPerThread" */ 'p', 'o', 'w', 'e', 'r', 'C', 'p', 'u', 'M', 's', 'P', 'e', 'r', 'T', 'h', 'r', 'e', 'a', 'd', '\0',
  /*   676 - "powerWakeupsPerThread" */ 'p', 'o', 'w', 'e', 'r', 'W', 'a', 'k', 'e', 'u', 'p', 's', 'P', 'e', 'r', 'T', 'h', 'r', 'e', 'a', 'd', '\0',
  /*   698 - "fingerprintingProtection" */ 'f', 'i', 'n', 'g', 'e', 'r', 'p', 'r', 'i', 'n', 't', 'i', 'n', 'g', 'P', 'r', 'o', 't', 'e', 'c', 't', 'i', 'o', 'n', '\0',
  /*   723 - "searchEngineDefault" */ 's', 'e', 'a', 'r', 'c', 'h', 'E', 'n', 'g', 'i', 'n', 'e', 'D', 'e', 'f', 'a', 'u', 'l', 't', '\0',
  /*   743 - "searchEnginePrivate" */ 's', 'e', 'a', 'r', 'c', 'h', 'E', 'n', 'g', 'i', 'n', 'e', 'P', 'r', 'i', 'v', 'a', 't', 'e', '\0',
  /*   763 - "searchService" */ 's', 'e', 'a', 'r', 'c', 'h', 'S', 'e', 'r', 'v', 'i', 'c', 'e', '\0',
  /*   777 - "dap" */ 'd', 'a', 'p', '\0',
  /*   781 - "legacyTelemetry" */ 'l', 'e', 'g', 'a', 'c', 'y', 'T', 'e', 'l', 'e', 'm', 'e', 't', 'r', 'y', '\0',
  /*   797 - "translations" */ 't', 'r', 'a', 'n', 's', 'l', 'a', 't', 'i', 'o', 'n', 's', '\0',
  /*   810 - "translationsPanel" */ 't', 'r', 'a', 'n', 's', 'l', 'a', 't', 'i', 'o', 'n', 's', 'P', 'a', 'n', 'e', 'l', '\0',
  /*   828 - "firstStartup" */ 'f', 'i', 'r', 's', 't', 'S', 't', 'a', 'r', 't', 'u', 'p', '\0',
  /*   841 - "blocklist" */ 'b', 'l', 'o', 'c', 'k', 'l', 'i', 's', 't', '\0',
  /*   851 - "update" */ 'u', 'p', 'd', 'a', 't', 'e', '\0',
  /*   858 - "timerThread" */ 't', 'i', 'm', 'e', 'r', 'T', 'h', 'r', 'e', 'a', 'd', '\0',
};


static_assert(sizeof(gCategoryStringTable) < UINT32_MAX, "Category string table is too large.");

const category_entry_t sCategoryByNameLookupEntries[] = {
  851ul,
  344ul,
  108ul,
  263ul,
  58ul,
  210ul,
  164ul,
  128ul,
  558ul,
  797ul,
  656ul,
  121ul,
  625ul,
  325ul,
  631ul,
  600ul,
  313ul,
  763ul,
  302ul,
  637ul,
  276ul,
  466ul,
  248ul,
  698ul,
  352ul,
  781ul,
  676ul,
  588ul,
  184ul,
  777ul,
  336ul,
  377ul,
  146ul,
  407ul,
  810ul,
  223ul,
  501ul,
  179ul,
  447ul,
  472ul,
  17ul,
  723ul,
  101ul,
  316ul,
  565ul,
  579ul,
  137ul,
  841ul,
  483ul,
  643ul,
  433ul,
  307ul,
  230ul,
  612ul,
  858ul,
  0ul,
  74ul,
  828ul,
  554ul,
  743ul,
  193ul,
  41ul,
  259ul,
  530ul
};



Maybe<uint32_t>
CategoryByNameLookup(const nsACString& aKey)
{
  static const uint8_t BASES[] = {
       0,   0,   0,   0,   0,   0,   0,   2,   3,   0,   0,   0,   1,   0,   0,   0,
       1,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   2,   0,   0,   0,
       0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,
       0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   2,
       0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,
       0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,
       0,   0,   0,   0,   0,   0,   1,   0,   0,   0,   0,   0,   0,   0,   0,   3,
       0,   0,   0,   0,   1,   0,   5,   1,   0,   0,   0,   0,   0,   0,   0,   0,
       1,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   3,   0,   0,
       0,   0,   0,   0,   0,   0,   3,   0,   0,   0,   0,   0,   0,   0,   0,   0,
       0,   0,   0,   1,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   2,
       0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   1,   0,   0,   0,   0,   0,
       0,   0,   0,   0,   0,   1,   0,   0,   0,   0,   0,   1,   0,   0,   0,   2,
       0,   0,   0,   0,   3,   0,   0,   0,   0,   0,   0,   0,   0,   0,   1,   0,
       1,   1,   0,   4,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,
       0,   0,   0,   2,   0,   0,   3,   0,   0,   0,   0,   3,   0,   0,   0,   0,
       0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,
       0,   2,   2,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,
       6,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,
       0,   0,   0,   5,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,
       0,   0,   0,   0,   0,   0,   2,   0,   0,   0,   6,   0,   0,   0,   0,   0,
       0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   2,   0,
       0,   0,   0,   1,   0,   0,   0,   1,   0,   0,   0,   0,   0,   0,   0,   0,
       0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,
       0,   0,   3,   0,   0,   0,   0,   1,   0,   0,   0,   0,   0,   1,   3,   0,
       0,   0,   0,   6,   0,   0,   0,   0,   0,   0,   1,   0,   0,   4,   0,   0,
       0,   0,   0,   0,   0,   0,   2,   0,   0,   0,   0,   0,   6,   0,   0,   0,
       0,   0,   8,   0,   0,   0,   0,   0,   0,   0,   0,   0,   3,   0,   0,   0,
       0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,  13,   0,
       0,   0,   8,   0,   0,   1,   0,   0,   2,   0,   0,   0,   0,   0,   0,  19,
       0,   0,   0,   0,   0,   0,   7,   0,   0,   0,   0,   0,   0,   0,   0,   0,
       0,  18,   0,   0,   0,   0,   0,  39,   0,   0,   0,   0,   0,  24,   0,   0,
  };
  

  const char* bytes = aKey.BeginReading();
  size_t length = aKey.Length();
  auto& entry = mozilla::perfecthash::Lookup(bytes, length, BASES,
                                             sCategoryByNameLookupEntries);
  return category_result_check(aKey, entry);
}


#if defined(_MSC_VER) && !defined(__clang__)
const char gMetricStringTable[] = {
#else
constexpr char gMetricStringTable[] = {
#endif
  /*     0 - "backgroundUpdate.reasonsToNotUpdate" */ 'b', 'a', 'c', 'k', 'g', 'r', 'o', 'u', 'n', 'd', 'U', 'p', 'd', 'a', 't', 'e', '.', 'r', 'e', 'a', 's', 'o', 'n', 's', 'T', 'o', 'N', 'o', 't', 'U', 'p', 'd', 'a', 't', 'e', '\0',
  /*    36 - "backgroundUpdate.timeLastUpdateScheduled" */ 'b', 'a', 'c', 'k', 'g', 'r', 'o', 'u', 'n', 'd', 'U', 'p', 'd', 'a', 't', 'e', '.', 't', 'i', 'm', 'e', 'L', 'a', 's', 't', 'U', 'p', 'd', 'a', 't', 'e', 'S', 'c', 'h', 'e', 'd', 'u', 'l', 'e', 'd', '\0',
  /*    77 - "backgroundUpdate.clientId" */ 'b', 'a', 'c', 'k', 'g', 'r', 'o', 'u', 'n', 'd', 'U', 'p', 'd', 'a', 't', 'e', '.', 'c', 'l', 'i', 'e', 'n', 't', 'I', 'd', '\0',
  /*   103 - "backgroundUpdate.exitCodeException" */ 'b', 'a', 'c', 'k', 'g', 'r', 'o', 'u', 'n', 'd', 'U', 'p', 'd', 'a', 't', 'e', '.', 'e', 'x', 'i', 't', 'C', 'o', 'd', 'e', 'E', 'x', 'c', 'e', 'p', 't', 'i', 'o', 'n', '\0',
  /*   138 - "backgroundUpdate.exitCodeSuccess" */ 'b', 'a', 'c', 'k', 'g', 'r', 'o', 'u', 'n', 'd', 'U', 'p', 'd', 'a', 't', 'e', '.', 'e', 'x', 'i', 't', 'C', 'o', 'd', 'e', 'S', 'u', 'c', 'c', 'e', 's', 's', '\0',
  /*   171 - "backgroundUpdate.finalState" */ 'b', 'a', 'c', 'k', 'g', 'r', 'o', 'u', 'n', 'd', 'U', 'p', 'd', 'a', 't', 'e', '.', 'f', 'i', 'n', 'a', 'l', 'S', 't', 'a', 't', 'e', '\0',
  /*   199 - "backgroundUpdate.reasons" */ 'b', 'a', 'c', 'k', 'g', 'r', 'o', 'u', 'n', 'd', 'U', 'p', 'd', 'a', 't', 'e', '.', 'r', 'e', 'a', 's', 'o', 'n', 's', '\0',
  /*   224 - "backgroundUpdate.registeredRestartAttempted" */ 'b', 'a', 'c', 'k', 'g', 'r', 'o', 'u', 'n', 'd', 'U', 'p', 'd', 'a', 't', 'e', '.', 'r', 'e', 'g', 'i', 's', 't', 'e', 'r', 'e', 'd', 'R', 'e', 's', 't', 'a', 'r', 't', 'A', 't', 't', 'e', 'm', 'p', 't', 'e', 'd', '\0',
  /*   268 - "backgroundUpdate.registeredRestartSuccess" */ 'b', 'a', 'c', 'k', 'g', 'r', 'o', 'u', 'n', 'd', 'U', 'p', 'd', 'a', 't', 'e', '.', 'r', 'e', 'g', 'i', 's', 't', 'e', 'r', 'e', 'd', 'R', 'e', 's', 't', 'a', 'r', 't', 'S', 'u', 'c', 'c', 'e', 's', 's', '\0',
  /*   310 - "backgroundUpdate.states" */ 'b', 'a', 'c', 'k', 'g', 'r', 'o', 'u', 'n', 'd', 'U', 'p', 'd', 'a', 't', 'e', '.', 's', 't', 'a', 't', 'e', 's', '\0',
  /*   334 - "backgroundUpdate.targetingEnvCurrentDate" */ 'b', 'a', 'c', 'k', 'g', 'r', 'o', 'u', 'n', 'd', 'U', 'p', 'd', 'a', 't', 'e', '.', 't', 'a', 'r', 'g', 'e', 't', 'i', 'n', 'g', 'E', 'n', 'v', 'C', 'u', 'r', 'r', 'e', 'n', 't', 'D', 'a', 't', 'e', '\0',
  /*   375 - "backgroundUpdate.targetingEnvFirefoxVersion" */ 'b', 'a', 'c', 'k', 'g', 'r', 'o', 'u', 'n', 'd', 'U', 'p', 'd', 'a', 't', 'e', '.', 't', 'a', 'r', 'g', 'e', 't', 'i', 'n', 'g', 'E', 'n', 'v', 'F', 'i', 'r', 'e', 'f', 'o', 'x', 'V', 'e', 'r', 's', 'i', 'o', 'n', '\0',
  /*   419 - "backgroundUpdate.targetingEnvProfileAge" */ 'b', 'a', 'c', 'k', 'g', 'r', 'o', 'u', 'n', 'd', 'U', 'p', 'd', 'a', 't', 'e', '.', 't', 'a', 'r', 'g', 'e', 't', 'i', 'n', 'g', 'E', 'n', 'v', 'P', 'r', 'o', 'f', 'i', 'l', 'e', 'A', 'g', 'e', '\0',
  /*   459 - "backgroundUpdate.targetingException" */ 'b', 'a', 'c', 'k', 'g', 'r', 'o', 'u', 'n', 'd', 'U', 'p', 'd', 'a', 't', 'e', '.', 't', 'a', 'r', 'g', 'e', 't', 'i', 'n', 'g', 'E', 'x', 'c', 'e', 'p', 't', 'i', 'o', 'n', '\0',
  /*   495 - "backgroundUpdate.targetingExists" */ 'b', 'a', 'c', 'k', 'g', 'r', 'o', 'u', 'n', 'd', 'U', 'p', 'd', 'a', 't', 'e', '.', 't', 'a', 'r', 'g', 'e', 't', 'i', 'n', 'g', 'E', 'x', 'i', 's', 't', 's', '\0',
  /*   528 - "backgroundUpdate.targetingVersion" */ 'b', 'a', 'c', 'k', 'g', 'r', 'o', 'u', 'n', 'd', 'U', 'p', 'd', 'a', 't', 'e', '.', 't', 'a', 'r', 'g', 'e', 't', 'i', 'n', 'g', 'V', 'e', 'r', 's', 'i', 'o', 'n', '\0',
  /*   562 - "browserLaunchedToHandle.systemNotification" */ 'b', 'r', 'o', 'w', 's', 'e', 'r', 'L', 'a', 'u', 'n', 'c', 'h', 'e', 'd', 'T', 'o', 'H', 'a', 'n', 'd', 'l', 'e', '.', 's', 'y', 's', 't', 'e', 'm', 'N', 'o', 't', 'i', 'f', 'i', 'c', 'a', 't', 'i', 'o', 'n', '\0',
  /*   605 - "browserMigration.matchedExtensions" */ 'b', 'r', 'o', 'w', 's', 'e', 'r', 'M', 'i', 'g', 'r', 'a', 't', 'i', 'o', 'n', '.', 'm', 'a', 't', 'c', 'h', 'e', 'd', 'E', 'x', 't', 'e', 'n', 's', 'i', 'o', 'n', 's', '\0',
  /*   640 - "browserMigration.unmatchedExtensions" */ 'b', 'r', 'o', 'w', 's', 'e', 'r', 'M', 'i', 'g', 'r', 'a', 't', 'i', 'o', 'n', '.', 'u', 'n', 'm', 'a', 't', 'c', 'h', 'e', 'd', 'E', 'x', 't', 'e', 'n', 's', 'i', 'o', 'n', 's', '\0',
  /*   677 - "messagingSystem.action" */ 'm', 'e', 's', 's', 'a', 'g', 'i', 'n', 'g', 'S', 'y', 's', 't', 'e', 'm', '.', 'a', 'c', 't', 'i', 'o', 'n', '\0',
  /*   700 - "messagingSystem.addonVersion" */ 'm', 'e', 's', 's', 'a', 'g', 'i', 'n', 'g', 'S', 'y', 's', 't', 'e', 'm', '.', 'a', 'd', 'd', 'o', 'n', 'V', 'e', 'r', 's', 'i', 'o', 'n', '\0',
  /*   729 - "messagingSystem.browserSessionId" */ 'm', 'e', 's', 's', 'a', 'g', 'i', 'n', 'g', 'S', 'y', 's', 't', 'e', 'm', '.', 'b', 'r', 'o', 'w', 's', 'e', 'r', 'S', 'e', 's', 's', 'i', 'o', 'n', 'I', 'd', '\0',
  /*   762 - "messagingSystem.bucketId" */ 'm', 'e', 's', 's', 'a', 'g', 'i', 'n', 'g', 'S', 'y', 's', 't', 'e', 'm', '.', 'b', 'u', 'c', 'k', 'e', 't', 'I', 'd', '\0',
  /*   787 - "messagingSystem.cfrAction" */ 'm', 'e', 's', 's', 'a', 'g', 'i', 'n', 'g', 'S', 'y', 's', 't', 'e', 'm', '.', 'c', 'f', 'r', 'A', 'c', 't', 'i', 'o', 'n', '\0',
  /*   813 - "messagingSystem.clientId" */ 'm', 'e', 's', 's', 'a', 'g', 'i', 'n', 'g', 'S', 'y', 's', 't', 'e', 'm', '.', 'c', 'l', 'i', 'e', 'n', 't', 'I', 'd', '\0',
  /*   838 - "messagingSystem.event" */ 'm', 'e', 's', 's', 'a', 'g', 'i', 'n', 'g', 'S', 'y', 's', 't', 'e', 'm', '.', 'e', 'v', 'e', 'n', 't', '\0',
  /*   860 - "messagingSystem.eventContext" */ 'm', 'e', 's', 's', 'a', 'g', 'i', 'n', 'g', 'S', 'y', 's', 't', 'e', 'm', '.', 'e', 'v', 'e', 'n', 't', 'C', 'o', 'n', 't', 'e', 'x', 't', '\0',
  /*   889 - "messagingSystem.eventContextParseError" */ 'm', 'e', 's', 's', 'a', 'g', 'i', 'n', 'g', 'S', 'y', 's', 't', 'e', 'm', '.', 'e', 'v', 'e', 'n', 't', 'C', 'o', 'n', 't', 'e', 'x', 't', 'P', 'a', 'r', 's', 'e', 'E', 'r', 'r', 'o', 'r', '\0',
  /*   928 - "messagingSystem.eventPage" */ 'm', 'e', 's', 's', 'a', 'g', 'i', 'n', 'g', 'S', 'y', 's', 't', 'e', 'm', '.', 'e', 'v', 'e', 'n', 't', 'P', 'a', 'g', 'e', '\0',
  /*   954 - "messagingSystem.eventReason" */ 'm', 'e', 's', 's', 'a', 'g', 'i', 'n', 'g', 'S', 'y', 's', 't', 'e', 'm', '.', 'e', 'v', 'e', 'n', 't', 'R', 'e', 'a', 's', 'o', 'n', '\0',
  /*   982 - "messagingSystem.eventSource" */ 'm', 'e', 's', 's', 'a', 'g', 'i', 'n', 'g', 'S', 'y', 's', 't', 'e', 'm', '.', 'e', 'v', 'e', 'n', 't', 'S', 'o', 'u', 'r', 'c', 'e', '\0',
  /*  1010 - "messagingSystem.gleanPingForPingFailures" */ 'm', 'e', 's', 's', 'a', 'g', 'i', 'n', 'g', 'S', 'y', 's', 't', 'e', 'm', '.', 'g', 'l', 'e', 'a', 'n', 'P', 'i', 'n', 'g', 'F', 'o', 'r', 'P', 'i', 'n', 'g', 'F', 'a', 'i', 'l', 'u', 'r', 'e', 's', '\0',
  /*  1051 - "messagingSystem.impressionId" */ 'm', 'e', 's', 's', 'a', 'g', 'i', 'n', 'g', 'S', 'y', 's', 't', 'e', 'm', '.', 'i', 'm', 'p', 'r', 'e', 's', 's', 'i', 'o', 'n', 'I', 'd', '\0',
  /*  1080 - "messagingSystem.invalidNestedData" */ 'm', 'e', 's', 's', 'a', 'g', 'i', 'n', 'g', 'S', 'y', 's', 't', 'e', 'm', '.', 'i', 'n', 'v', 'a', 'l', 'i', 'd', 'N', 'e', 's', 't', 'e', 'd', 'D', 'a', 't', 'a', '\0',
  /*  1114 - "messagingSystem.locale" */ 'm', 'e', 's', 's', 'a', 'g', 'i', 'n', 'g', 'S', 'y', 's', 't', 'e', 'm', '.', 'l', 'o', 'c', 'a', 'l', 'e', '\0',
  /*  1137 - "messagingSystem.messageId" */ 'm', 'e', 's', 's', 'a', 'g', 'i', 'n', 'g', 'S', 'y', 's', 't', 'e', 'm', '.', 'm', 'e', 's', 's', 'a', 'g', 'e', 'I', 'd', '\0',
  /*  1163 - "messagingSystem.page" */ 'm', 'e', 's', 's', 'a', 'g', 'i', 'n', 'g', 'S', 'y', 's', 't', 'e', 'm', '.', 'p', 'a', 'g', 'e', '\0',
  /*  1184 - "messagingSystem.pingType" */ 'm', 'e', 's', 's', 'a', 'g', 'i', 'n', 'g', 'S', 'y', 's', 't', 'e', 'm', '.', 'p', 'i', 'n', 'g', 'T', 'y', 'p', 'e', '\0',
  /*  1209 - "messagingSystem.source" */ 'm', 'e', 's', 's', 'a', 'g', 'i', 'n', 'g', 'S', 'y', 's', 't', 'e', 'm', '.', 's', 'o', 'u', 'r', 'c', 'e', '\0',
  /*  1232 - "messagingSystem.unknownKeyCount" */ 'm', 'e', 's', 's', 'a', 'g', 'i', 'n', 'g', 'S', 'y', 's', 't', 'e', 'm', '.', 'u', 'n', 'k', 'n', 'o', 'w', 'n', 'K', 'e', 'y', 'C', 'o', 'u', 'n', 't', '\0',
  /*  1264 - "messagingSystem.unknownKeys" */ 'm', 'e', 's', 's', 'a', 'g', 'i', 'n', 'g', 'S', 'y', 's', 't', 'e', 'm', '.', 'u', 'n', 'k', 'n', 'o', 'w', 'n', 'K', 'e', 'y', 's', '\0',
  /*  1292 - "messagingSystemAttribution.campaign" */ 'm', 'e', 's', 's', 'a', 'g', 'i', 'n', 'g', 'S', 'y', 's', 't', 'e', 'm', 'A', 't', 't', 'r', 'i', 'b', 'u', 't', 'i', 'o', 'n', '.', 'c', 'a', 'm', 'p', 'a', 'i', 'g', 'n', '\0',
  /*  1328 - "messagingSystemAttribution.content" */ 'm', 'e', 's', 's', 'a', 'g', 'i', 'n', 'g', 'S', 'y', 's', 't', 'e', 'm', 'A', 't', 't', 'r', 'i', 'b', 'u', 't', 'i', 'o', 'n', '.', 'c', 'o', 'n', 't', 'e', 'n', 't', '\0',
  /*  1363 - "messagingSystemAttribution.dlsource" */ 'm', 'e', 's', 's', 'a', 'g', 'i', 'n', 'g', 'S', 'y', 's', 't', 'e', 'm', 'A', 't', 't', 'r', 'i', 'b', 'u', 't', 'i', 'o', 'n', '.', 'd', 'l', 's', 'o', 'u', 'r', 'c', 'e', '\0',
  /*  1399 - "messagingSystemAttribution.dltoken" */ 'm', 'e', 's', 's', 'a', 'g', 'i', 'n', 'g', 'S', 'y', 's', 't', 'e', 'm', 'A', 't', 't', 'r', 'i', 'b', 'u', 't', 'i', 'o', 'n', '.', 'd', 'l', 't', 'o', 'k', 'e', 'n', '\0',
  /*  1434 - "messagingSystemAttribution.experiment" */ 'm', 'e', 's', 's', 'a', 'g', 'i', 'n', 'g', 'S', 'y', 's', 't', 'e', 'm', 'A', 't', 't', 'r', 'i', 'b', 'u', 't', 'i', 'o', 'n', '.', 'e', 'x', 'p', 'e', 'r', 'i', 'm', 'e', 'n', 't', '\0',
  /*  1472 - "messagingSystemAttribution.medium" */ 'm', 'e', 's', 's', 'a', 'g', 'i', 'n', 'g', 'S', 'y', 's', 't', 'e', 'm', 'A', 't', 't', 'r', 'i', 'b', 'u', 't', 'i', 'o', 'n', '.', 'm', 'e', 'd', 'i', 'u', 'm', '\0',
  /*  1506 - "messagingSystemAttribution.msstoresignedin" */ 'm', 'e', 's', 's', 'a', 'g', 'i', 'n', 'g', 'S', 'y', 's', 't', 'e', 'm', 'A', 't', 't', 'r', 'i', 'b', 'u', 't', 'i', 'o', 'n', '.', 'm', 's', 's', 't', 'o', 'r', 'e', 's', 'i', 'g', 'n', 'e', 'd', 'i', 'n', '\0',
  /*  1549 - "messagingSystemAttribution.source" */ 'm', 'e', 's', 's', 'a', 'g', 'i', 'n', 'g', 'S', 'y', 's', 't', 'e', 'm', 'A', 't', 't', 'r', 'i', 'b', 'u', 't', 'i', 'o', 'n', '.', 's', 'o', 'u', 'r', 'c', 'e', '\0',
  /*  1583 - "messagingSystemAttribution.ua" */ 'm', 'e', 's', 's', 'a', 'g', 'i', 'n', 'g', 'S', 'y', 's', 't', 'e', 'm', 'A', 't', 't', 'r', 'i', 'b', 'u', 't', 'i', 'o', 'n', '.', 'u', 'a', '\0',
  /*  1613 - "messagingSystemAttribution.unknownKeys" */ 'm', 'e', 's', 's', 'a', 'g', 'i', 'n', 'g', 'S', 'y', 's', 't', 'e', 'm', 'A', 't', 't', 'r', 'i', 'b', 'u', 't', 'i', 'o', 'n', '.', 'u', 'n', 'k', 'n', 'o', 'w', 'n', 'K', 'e', 'y', 's', '\0',
  /*  1652 - "messagingSystemAttribution.variation" */ 'm', 'e', 's', 's', 'a', 'g', 'i', 'n', 'g', 'S', 'y', 's', 't', 'e', 'm', 'A', 't', 't', 'r', 'i', 'b', 'u', 't', 'i', 'o', 'n', '.', 'v', 'a', 'r', 'i', 'a', 't', 'i', 'o', 'n', '\0',
  /*  1689 - "newtab.blockedSponsors" */ 'n', 'e', 'w', 't', 'a', 'b', '.', 'b', 'l', 'o', 'c', 'k', 'e', 'd', 'S', 'p', 'o', 'n', 's', 'o', 'r', 's', '\0',
  /*  1712 - "newtab.closed" */ 'n', 'e', 'w', 't', 'a', 'b', '.', 'c', 'l', 'o', 's', 'e', 'd', '\0',
  /*  1726 - "newtab.homepageCategory" */ 'n', 'e', 'w', 't', 'a', 'b', '.', 'h', 'o', 'm', 'e', 'p', 'a', 'g', 'e', 'C', 'a', 't', 'e', 'g', 'o', 'r', 'y', '\0',
  /*  1750 - "newtab.locale" */ 'n', 'e', 'w', 't', 'a', 'b', '.', 'l', 'o', 'c', 'a', 'l', 'e', '\0',
  /*  1764 - "newtab.newtabCategory" */ 'n', 'e', 'w', 't', 'a', 'b', '.', 'n', 'e', 'w', 't', 'a', 'b', 'C', 'a', 't', 'e', 'g', 'o', 'r', 'y', '\0',
  /*  1786 - "newtab.opened" */ 'n', 'e', 'w', 't', 'a', 'b', '.', 'o', 'p', 'e', 'n', 'e', 'd', '\0',
  /*  1800 - "newtab.sovAllocation" */ 'n', 'e', 'w', 't', 'a', 'b', '.', 's', 'o', 'v', 'A', 'l', 'l', 'o', 'c', 'a', 't', 'i', 'o', 'n', '\0',
  /*  1821 - "newtabSearch.enabled" */ 'n', 'e', 'w', 't', 'a', 'b', 'S', 'e', 'a', 'r', 'c', 'h', '.', 'e', 'n', 'a', 'b', 'l', 'e', 'd', '\0',
  /*  1842 - "newtabSearch.issued" */ 'n', 'e', 'w', 't', 'a', 'b', 'S', 'e', 'a', 'r', 'c', 'h', '.', 'i', 's', 's', 'u', 'e', 'd', '\0',
  /*  1862 - "pocket.click" */ 'p', 'o', 'c', 'k', 'e', 't', '.', 'c', 'l', 'i', 'c', 'k', '\0',
  /*  1875 - "pocket.enabled" */ 'p', 'o', 'c', 'k', 'e', 't', '.', 'e', 'n', 'a', 'b', 'l', 'e', 'd', '\0',
  /*  1890 - "pocket.impression" */ 'p', 'o', 'c', 'k', 'e', 't', '.', 'i', 'm', 'p', 'r', 'e', 's', 's', 'i', 'o', 'n', '\0',
  /*  1908 - "pocket.isSignedIn" */ 'p', 'o', 'c', 'k', 'e', 't', '.', 'i', 's', 'S', 'i', 'g', 'n', 'e', 'd', 'I', 'n', '\0',
  /*  1926 - "pocket.save" */ 'p', 'o', 'c', 'k', 'e', 't', '.', 's', 'a', 'v', 'e', '\0',
  /*  1938 - "pocket.sponsoredStoriesEnabled" */ 'p', 'o', 'c', 'k', 'e', 't', '.', 's', 'p', 'o', 'n', 's', 'o', 'r', 'e', 'd', 'S', 't', 'o', 'r', 'i', 'e', 's', 'E', 'n', 'a', 'b', 'l', 'e', 'd', '\0',
  /*  1969 - "pocket.topicClick" */ 'p', 'o', 'c', 'k', 'e', 't', '.', 't', 'o', 'p', 'i', 'c', 'C', 'l', 'i', 'c', 'k', '\0',
  /*  1987 - "topSites.advertiser" */ 't', 'o', 'p', 'S', 'i', 't', 'e', 's', '.', 'a', 'd', 'v', 'e', 'r', 't', 'i', 's', 'e', 'r', '\0',
  /*  2007 - "topSites.contextId" */ 't', 'o', 'p', 'S', 'i', 't', 'e', 's', '.', 'c', 'o', 'n', 't', 'e', 'x', 't', 'I', 'd', '\0',
  /*  2026 - "topSites.pingType" */ 't', 'o', 'p', 'S', 'i', 't', 'e', 's', '.', 'p', 'i', 'n', 'g', 'T', 'y', 'p', 'e', '\0',
  /*  2044 - "topSites.position" */ 't', 'o', 'p', 'S', 'i', 't', 'e', 's', '.', 'p', 'o', 's', 'i', 't', 'i', 'o', 'n', '\0',
  /*  2062 - "topSites.reportingUrl" */ 't', 'o', 'p', 'S', 'i', 't', 'e', 's', '.', 'r', 'e', 'p', 'o', 'r', 't', 'i', 'n', 'g', 'U', 'r', 'l', '\0',
  /*  2084 - "topSites.source" */ 't', 'o', 'p', 'S', 'i', 't', 'e', 's', '.', 's', 'o', 'u', 'r', 'c', 'e', '\0',
  /*  2100 - "topSites.tileId" */ 't', 'o', 'p', 'S', 'i', 't', 'e', 's', '.', 't', 'i', 'l', 'e', 'I', 'd', '\0',
  /*  2116 - "topsites.click" */ 't', 'o', 'p', 's', 'i', 't', 'e', 's', '.', 'c', 'l', 'i', 'c', 'k', '\0',
  /*  2131 - "topsites.enabled" */ 't', 'o', 'p', 's', 'i', 't', 'e', 's', '.', 'e', 'n', 'a', 'b', 'l', 'e', 'd', '\0',
  /*  2148 - "topsites.impression" */ 't', 'o', 'p', 's', 'i', 't', 'e', 's', '.', 'i', 'm', 'p', 'r', 'e', 's', 's', 'i', 'o', 'n', '\0',
  /*  2168 - "topsites.rows" */ 't', 'o', 'p', 's', 'i', 't', 'e', 's', '.', 'r', 'o', 'w', 's', '\0',
  /*  2182 - "topsites.sponsoredEnabled" */ 't', 'o', 'p', 's', 'i', 't', 'e', 's', '.', 's', 'p', 'o', 'n', 's', 'o', 'r', 'e', 'd', 'E', 'n', 'a', 'b', 'l', 'e', 'd', '\0',
  /*  2208 - "privacyUiFppClick.checkbox" */ 'p', 'r', 'i', 'v', 'a', 'c', 'y', 'U', 'i', 'F', 'p', 'p', 'C', 'l', 'i', 'c', 'k', '.', 'c', 'h', 'e', 'c', 'k', 'b', 'o', 'x', '\0',
  /*  2235 - "privacyUiFppClick.menu" */ 'p', 'r', 'i', 'v', 'a', 'c', 'y', 'U', 'i', 'F', 'p', 'p', 'C', 'l', 'i', 'c', 'k', '.', 'm', 'e', 'n', 'u', '\0',
  /*  2258 - "newtabSearchAd.click" */ 'n', 'e', 'w', 't', 'a', 'b', 'S', 'e', 'a', 'r', 'c', 'h', 'A', 'd', '.', 'c', 'l', 'i', 'c', 'k', '\0',
  /*  2279 - "newtabSearchAd.impression" */ 'n', 'e', 'w', 't', 'a', 'b', 'S', 'e', 'a', 'r', 'c', 'h', 'A', 'd', '.', 'i', 'm', 'p', 'r', 'e', 's', 's', 'i', 'o', 'n', '\0',
  /*  2305 - "serp.abandonment" */ 's', 'e', 'r', 'p', '.', 'a', 'b', 'a', 'n', 'd', 'o', 'n', 'm', 'e', 'n', 't', '\0',
  /*  2322 - "serp.adImpression" */ 's', 'e', 'r', 'p', '.', 'a', 'd', 'I', 'm', 'p', 'r', 'e', 's', 's', 'i', 'o', 'n', '\0',
  /*  2340 - "serp.categorizationDuration" */ 's', 'e', 'r', 'p', '.', 'c', 'a', 't', 'e', 'g', 'o', 'r', 'i', 'z', 'a', 't', 'i', 'o', 'n', 'D', 'u', 'r', 'a', 't', 'i', 'o', 'n', '\0',
  /*  2368 - "serp.engagement" */ 's', 'e', 'r', 'p', '.', 'e', 'n', 'g', 'a', 'g', 'e', 'm', 'e', 'n', 't', '\0',
  /*  2384 - "serp.impression" */ 's', 'e', 'r', 'p', '.', 'i', 'm', 'p', 'r', 'e', 's', 's', 'i', 'o', 'n', '\0',
  /*  2400 - "shopping.surfaceReanalyzeClicked" */ 's', 'h', 'o', 'p', 'p', 'i', 'n', 'g', '.', 's', 'u', 'r', 'f', 'a', 'c', 'e', 'R', 'e', 'a', 'n', 'a', 'l', 'y', 'z', 'e', 'C', 'l', 'i', 'c', 'k', 'e', 'd', '\0',
  /*  2433 - "shopping.surfaceSettingsExpandClicked" */ 's', 'h', 'o', 'p', 'p', 'i', 'n', 'g', '.', 's', 'u', 'r', 'f', 'a', 'c', 'e', 'S', 'e', 't', 't', 'i', 'n', 'g', 's', 'E', 'x', 'p', 'a', 'n', 'd', 'C', 'l', 'i', 'c', 'k', 'e', 'd', '\0',
  /*  2471 - "shopping.surfaceShowQualityExplainerClicked" */ 's', 'h', 'o', 'p', 'p', 'i', 'n', 'g', '.', 's', 'u', 'r', 'f', 'a', 'c', 'e', 'S', 'h', 'o', 'w', 'Q', 'u', 'a', 'l', 'i', 't', 'y', 'E', 'x', 'p', 'l', 'a', 'i', 'n', 'e', 'r', 'C', 'l', 'i', 'c', 'k', 'e', 'd', '\0',
  /*  2515 - "shoppingSettings.componentOptedOut" */ 's', 'h', 'o', 'p', 'p', 'i', 'n', 'g', 'S', 'e', 't', 't', 'i', 'n', 'g', 's', '.', 'c', 'o', 'm', 'p', 'o', 'n', 'e', 'n', 't', 'O', 'p', 't', 'e', 'd', 'O', 'u', 't', '\0',
  /*  2550 - "shoppingSettings.hasOnboarded" */ 's', 'h', 'o', 'p', 'p', 'i', 'n', 'g', 'S', 'e', 't', 't', 'i', 'n', 'g', 's', '.', 'h', 'a', 's', 'O', 'n', 'b', 'o', 'a', 'r', 'd', 'e', 'd', '\0',
  /*  2580 - "shoppingSettings.nimbusDisabledShopping" */ 's', 'h', 'o', 'p', 'p', 'i', 'n', 'g', 'S', 'e', 't', 't', 'i', 'n', 'g', 's', '.', 'n', 'i', 'm', 'b', 'u', 's', 'D', 'i', 's', 'a', 'b', 'l', 'e', 'd', 'S', 'h', 'o', 'p', 'p', 'i', 'n', 'g', '\0',
  /*  2620 - "quickSuggest.advertiser" */ 'q', 'u', 'i', 'c', 'k', 'S', 'u', 'g', 'g', 'e', 's', 't', '.', 'a', 'd', 'v', 'e', 'r', 't', 'i', 's', 'e', 'r', '\0',
  /*  2644 - "quickSuggest.blockId" */ 'q', 'u', 'i', 'c', 'k', 'S', 'u', 'g', 'g', 'e', 's', 't', '.', 'b', 'l', 'o', 'c', 'k', 'I', 'd', '\0',
  /*  2665 - "quickSuggest.contextId" */ 'q', 'u', 'i', 'c', 'k', 'S', 'u', 'g', 'g', 'e', 's', 't', '.', 'c', 'o', 'n', 't', 'e', 'x', 't', 'I', 'd', '\0',
  /*  2688 - "quickSuggest.iabCategory" */ 'q', 'u', 'i', 'c', 'k', 'S', 'u', 'g', 'g', 'e', 's', 't', '.', 'i', 'a', 'b', 'C', 'a', 't', 'e', 'g', 'o', 'r', 'y', '\0',
  /*  2713 - "quickSuggest.improveSuggestExperience" */ 'q', 'u', 'i', 'c', 'k', 'S', 'u', 'g', 'g', 'e', 's', 't', '.', 'i', 'm', 'p', 'r', 'o', 'v', 'e', 'S', 'u', 'g', 'g', 'e', 's', 't', 'E', 'x', 'p', 'e', 'r', 'i', 'e', 'n', 'c', 'e', '\0',
  /*  2751 - "quickSuggest.isClicked" */ 'q', 'u', 'i', 'c', 'k', 'S', 'u', 'g', 'g', 'e', 's', 't', '.', 'i', 's', 'C', 'l', 'i', 'c', 'k', 'e', 'd', '\0',
  /*  2774 - "quickSuggest.matchType" */ 'q', 'u', 'i', 'c', 'k', 'S', 'u', 'g', 'g', 'e', 's', 't', '.', 'm', 'a', 't', 'c', 'h', 'T', 'y', 'p', 'e', '\0',
  /*  2797 - "quickSuggest.pingType" */ 'q', 'u', 'i', 'c', 'k', 'S', 'u', 'g', 'g', 'e', 's', 't', '.', 'p', 'i', 'n', 'g', 'T', 'y', 'p', 'e', '\0',
  /*  2819 - "quickSuggest.position" */ 'q', 'u', 'i', 'c', 'k', 'S', 'u', 'g', 'g', 'e', 's', 't', '.', 'p', 'o', 's', 'i', 't', 'i', 'o', 'n', '\0',
  /*  2841 - "quickSuggest.reportingUrl" */ 'q', 'u', 'i', 'c', 'k', 'S', 'u', 'g', 'g', 'e', 's', 't', '.', 'r', 'e', 'p', 'o', 'r', 't', 'i', 'n', 'g', 'U', 'r', 'l', '\0',
  /*  2867 - "quickSuggest.requestId" */ 'q', 'u', 'i', 'c', 'k', 'S', 'u', 'g', 'g', 'e', 's', 't', '.', 'r', 'e', 'q', 'u', 'e', 's', 't', 'I', 'd', '\0',
  /*  2890 - "quickSuggest.source" */ 'q', 'u', 'i', 'c', 'k', 'S', 'u', 'g', 'g', 'e', 's', 't', '.', 's', 'o', 'u', 'r', 'c', 'e', '\0',
  /*  2910 - "urlbar.abandonment" */ 'u', 'r', 'l', 'b', 'a', 'r', '.', 'a', 'b', 'a', 'n', 'd', 'o', 'n', 'm', 'e', 'n', 't', '\0',
  /*  2929 - "urlbar.engagement" */ 'u', 'r', 'l', 'b', 'a', 'r', '.', 'e', 'n', 'g', 'a', 'g', 'e', 'm', 'e', 'n', 't', '\0',
  /*  2947 - "urlbar.exposure" */ 'u', 'r', 'l', 'b', 'a', 'r', '.', 'e', 'x', 'p', 'o', 's', 'u', 'r', 'e', '\0',
  /*  2963 - "urlbar.impression" */ 'u', 'r', 'l', 'b', 'a', 'r', '.', 'i', 'm', 'p', 'r', 'e', 's', 's', 'i', 'o', 'n', '\0',
  /*  2981 - "urlbar.prefMaxResults" */ 'u', 'r', 'l', 'b', 'a', 'r', '.', 'p', 'r', 'e', 'f', 'M', 'a', 'x', 'R', 'e', 's', 'u', 'l', 't', 's', '\0',
  /*  3003 - "urlbar.prefSuggestDataCollection" */ 'u', 'r', 'l', 'b', 'a', 'r', '.', 'p', 'r', 'e', 'f', 'S', 'u', 'g', 'g', 'e', 's', 't', 'D', 'a', 't', 'a', 'C', 'o', 'l', 'l', 'e', 'c', 't', 'i', 'o', 'n', '\0',
  /*  3036 - "urlbar.prefSuggestNonsponsored" */ 'u', 'r', 'l', 'b', 'a', 'r', '.', 'p', 'r', 'e', 'f', 'S', 'u', 'g', 'g', 'e', 's', 't', 'N', 'o', 'n', 's', 'p', 'o', 'n', 's', 'o', 'r', 'e', 'd', '\0',
  /*  3067 - "urlbar.prefSuggestSponsored" */ 'u', 'r', 'l', 'b', 'a', 'r', '.', 'p', 'r', 'e', 'f', 'S', 'u', 'g', 'g', 'e', 's', 't', 'S', 'p', 'o', 'n', 's', 'o', 'r', 'e', 'd', '\0',
  /*  3095 - "urlbar.prefSuggestTopsites" */ 'u', 'r', 'l', 'b', 'a', 'r', '.', 'p', 'r', 'e', 'f', 'S', 'u', 'g', 'g', 'e', 's', 't', 'T', 'o', 'p', 's', 'i', 't', 'e', 's', '\0',
  /*  3122 - "browserEngagement.activeTicks" */ 'b', 'r', 'o', 'w', 's', 'e', 'r', 'E', 'n', 'g', 'a', 'g', 'e', 'm', 'e', 'n', 't', '.', 'a', 'c', 't', 'i', 'v', 'e', 'T', 'i', 'c', 'k', 's', '\0',
  /*  3152 - "browserEngagement.profileCount" */ 'b', 'r', 'o', 'w', 's', 'e', 'r', 'E', 'n', 'g', 'a', 'g', 'e', 'm', 'e', 'n', 't', '.', 'p', 'r', 'o', 'f', 'i', 'l', 'e', 'C', 'o', 'u', 'n', 't', '\0',
  /*  3183 - "browserEngagement.uriCount" */ 'b', 'r', 'o', 'w', 's', 'e', 'r', 'E', 'n', 'g', 'a', 'g', 'e', 'm', 'e', 'n', 't', '.', 'u', 'r', 'i', 'C', 'o', 'u', 'n', 't', '\0',
  /*  3210 - "pingCentre.sendFailures" */ 'p', 'i', 'n', 'g', 'C', 'e', 'n', 't', 'r', 'e', '.', 's', 'e', 'n', 'd', 'F', 'a', 'i', 'l', 'u', 'r', 'e', 's', '\0',
  /*  3234 - "pingCentre.sendFailuresByNamespace" */ 'p', 'i', 'n', 'g', 'C', 'e', 'n', 't', 'r', 'e', '.', 's', 'e', 'n', 'd', 'F', 'a', 'i', 'l', 'u', 'r', 'e', 's', 'B', 'y', 'N', 'a', 'm', 'e', 's', 'p', 'a', 'c', 'e', '\0',
  /*  3269 - "pingCentre.sendSuccessesByNamespace" */ 'p', 'i', 'n', 'g', 'C', 'e', 'n', 't', 'r', 'e', '.', 's', 'e', 'n', 'd', 'S', 'u', 'c', 'c', 'e', 's', 's', 'e', 's', 'B', 'y', 'N', 'a', 'm', 'e', 's', 'p', 'a', 'c', 'e', '\0',
  /*  3305 - "gmp.updateXmlFetchResult" */ 'g', 'm', 'p', '.', 'u', 'p', 'd', 'a', 't', 'e', 'X', 'm', 'l', 'F', 'e', 't', 'c', 'h', 'R', 'e', 's', 'u', 'l', 't', '\0',
  /*  3330 - "rtcrtpsender.count" */ 'r', 't', 'c', 'r', 't', 'p', 's', 'e', 'n', 'd', 'e', 'r', '.', 'c', 'o', 'u', 'n', 't', '\0',
  /*  3349 - "rtcrtpsender.countSetparametersCompat" */ 'r', 't', 'c', 'r', 't', 'p', 's', 'e', 'n', 'd', 'e', 'r', '.', 'c', 'o', 'u', 'n', 't', 'S', 'e', 't', 'p', 'a', 'r', 'a', 'm', 'e', 't', 'e', 'r', 's', 'C', 'o', 'm', 'p', 'a', 't', '\0',
  /*  3387 - "rtcrtpsender.usedSendencodings" */ 'r', 't', 'c', 'r', 't', 'p', 's', 'e', 'n', 'd', 'e', 'r', '.', 'u', 's', 'e', 'd', 'S', 'e', 'n', 'd', 'e', 'n', 'c', 'o', 'd', 'i', 'n', 'g', 's', '\0',
  /*  3418 - "rtcrtpsenderSetparameters.blameLengthChanged" */ 'r', 't', 'c', 'r', 't', 'p', 's', 'e', 'n', 'd', 'e', 'r', 'S', 'e', 't', 'p', 'a', 'r', 'a', 'm', 'e', 't', 'e', 'r', 's', '.', 'b', 'l', 'a', 'm', 'e', 'L', 'e', 'n', 'g', 't', 'h', 'C', 'h', 'a', 'n', 'g', 'e', 'd', '\0',
  /*  3463 - "rtcrtpsenderSetparameters.blameNoGetparameters" */ 'r', 't', 'c', 'r', 't', 'p', 's', 'e', 'n', 'd', 'e', 'r', 'S', 'e', 't', 'p', 'a', 'r', 'a', 'm', 'e', 't', 'e', 'r', 's', '.', 'b', 'l', 'a', 'm', 'e', 'N', 'o', 'G', 'e', 't', 'p', 'a', 'r', 'a', 'm', 'e', 't', 'e', 'r', 's', '\0',
  /*  3510 - "rtcrtpsenderSetparameters.blameNoTransactionid" */ 'r', 't', 'c', 'r', 't', 'p', 's', 'e', 'n', 'd', 'e', 'r', 'S', 'e', 't', 'p', 'a', 'r', 'a', 'm', 'e', 't', 'e', 'r', 's', '.', 'b', 'l', 'a', 'm', 'e', 'N', 'o', 'T', 'r', 'a', 'n', 's', 'a', 'c', 't', 'i', 'o', 'n', 'i', 'd', '\0',
  /*  3557 - "rtcrtpsenderSetparameters.blameStaleTransactionid" */ 'r', 't', 'c', 'r', 't', 'p', 's', 'e', 'n', 'd', 'e', 'r', 'S', 'e', 't', 'p', 'a', 'r', 'a', 'm', 'e', 't', 'e', 'r', 's', '.', 'b', 'l', 'a', 'm', 'e', 'S', 't', 'a', 'l', 'e', 'T', 'r', 'a', 'n', 's', 'a', 'c', 't', 'i', 'o', 'n', 'i', 'd', '\0',
  /*  3607 - "rtcrtpsenderSetparameters.failLengthChanged" */ 'r', 't', 'c', 'r', 't', 'p', 's', 'e', 'n', 'd', 'e', 'r', 'S', 'e', 't', 'p', 'a', 'r', 'a', 'm', 'e', 't', 'e', 'r', 's', '.', 'f', 'a', 'i', 'l', 'L', 'e', 'n', 'g', 't', 'h', 'C', 'h', 'a', 'n', 'g', 'e', 'd', '\0',
  /*  3651 - "rtcrtpsenderSetparameters.failNoEncodings" */ 'r', 't', 'c', 'r', 't', 'p', 's', 'e', 'n', 'd', 'e', 'r', 'S', 'e', 't', 'p', 'a', 'r', 'a', 'm', 'e', 't', 'e', 'r', 's', '.', 'f', 'a', 'i', 'l', 'N', 'o', 'E', 'n', 'c', 'o', 'd', 'i', 'n', 'g', 's', '\0',
  /*  3693 - "rtcrtpsenderSetparameters.failNoGetparameters" */ 'r', 't', 'c', 'r', 't', 'p', 's', 'e', 'n', 'd', 'e', 'r', 'S', 'e', 't', 'p', 'a', 'r', 'a', 'm', 'e', 't', 'e', 'r', 's', '.', 'f', 'a', 'i', 'l', 'N', 'o', 'G', 'e', 't', 'p', 'a', 'r', 'a', 'm', 'e', 't', 'e', 'r', 's', '\0',
  /*  3739 - "rtcrtpsenderSetparameters.failNoTransactionid" */ 'r', 't', 'c', 'r', 't', 'p', 's', 'e', 'n', 'd', 'e', 'r', 'S', 'e', 't', 'p', 'a', 'r', 'a', 'm', 'e', 't', 'e', 'r', 's', '.', 'f', 'a', 'i', 'l', 'N', 'o', 'T', 'r', 'a', 'n', 's', 'a', 'c', 't', 'i', 'o', 'n', 'i', 'd', '\0',
  /*  3785 - "rtcrtpsenderSetparameters.failOther" */ 'r', 't', 'c', 'r', 't', 'p', 's', 'e', 'n', 'd', 'e', 'r', 'S', 'e', 't', 'p', 'a', 'r', 'a', 'm', 'e', 't', 'e', 'r', 's', '.', 'f', 'a', 'i', 'l', 'O', 't', 'h', 'e', 'r', '\0',
  /*  3821 - "rtcrtpsenderSetparameters.failRidChanged" */ 'r', 't', 'c', 'r', 't', 'p', 's', 'e', 'n', 'd', 'e', 'r', 'S', 'e', 't', 'p', 'a', 'r', 'a', 'm', 'e', 't', 'e', 'r', 's', '.', 'f', 'a', 'i', 'l', 'R', 'i', 'd', 'C', 'h', 'a', 'n', 'g', 'e', 'd', '\0',
  /*  3862 - "rtcrtpsenderSetparameters.failStaleTransactionid" */ 'r', 't', 'c', 'r', 't', 'p', 's', 'e', 'n', 'd', 'e', 'r', 'S', 'e', 't', 'p', 'a', 'r', 'a', 'm', 'e', 't', 'e', 'r', 's', '.', 'f', 'a', 'i', 'l', 'S', 't', 'a', 'l', 'e', 'T', 'r', 'a', 'n', 's', 'a', 'c', 't', 'i', 'o', 'n', 'i', 'd', '\0',
  /*  3911 - "rtcrtpsenderSetparameters.warnLengthChanged" */ 'r', 't', 'c', 'r', 't', 'p', 's', 'e', 'n', 'd', 'e', 'r', 'S', 'e', 't', 'p', 'a', 'r', 'a', 'm', 'e', 't', 'e', 'r', 's', '.', 'w', 'a', 'r', 'n', 'L', 'e', 'n', 'g', 't', 'h', 'C', 'h', 'a', 'n', 'g', 'e', 'd', '\0',
  /*  3955 - "rtcrtpsenderSetparameters.warnNoGetparameters" */ 'r', 't', 'c', 'r', 't', 'p', 's', 'e', 'n', 'd', 'e', 'r', 'S', 'e', 't', 'p', 'a', 'r', 'a', 'm', 'e', 't', 'e', 'r', 's', '.', 'w', 'a', 'r', 'n', 'N', 'o', 'G', 'e', 't', 'p', 'a', 'r', 'a', 'm', 'e', 't', 'e', 'r', 's', '\0',
  /*  4001 - "rtcrtpsenderSetparameters.warnNoTransactionid" */ 'r', 't', 'c', 'r', 't', 'p', 's', 'e', 'n', 'd', 'e', 'r', 'S', 'e', 't', 'p', 'a', 'r', 'a', 'm', 'e', 't', 'e', 'r', 's', '.', 'w', 'a', 'r', 'n', 'N', 'o', 'T', 'r', 'a', 'n', 's', 'a', 'c', 't', 'i', 'o', 'n', 'i', 'd', '\0',
  /*  4047 - "rtcrtpsenderSetparameters.warnStaleTransactionid" */ 'r', 't', 'c', 'r', 't', 'p', 's', 'e', 'n', 'd', 'e', 'r', 'S', 'e', 't', 'p', 'a', 'r', 'a', 'm', 'e', 't', 'e', 'r', 's', '.', 'w', 'a', 'r', 'n', 'S', 't', 'a', 'l', 'e', 'T', 'r', 'a', 'n', 's', 'a', 'c', 't', 'i', 'o', 'n', 'i', 'd', '\0',
  /*  4096 - "perf.pageLoad" */ 'p', 'e', 'r', 'f', '.', 'p', 'a', 'g', 'e', 'L', 'o', 'a', 'd', '\0',
  /*  4110 - "paint.buildDisplaylistTime" */ 'p', 'a', 'i', 'n', 't', '.', 'b', 'u', 'i', 'l', 'd', 'D', 'i', 's', 'p', 'l', 'a', 'y', 'l', 'i', 's', 't', 'T', 'i', 'm', 'e', '\0',
  /*  4137 - "wr.framebuildTime" */ 'w', 'r', '.', 'f', 'r', 'a', 'm', 'e', 'b', 'u', 'i', 'l', 'd', 'T', 'i', 'm', 'e', '\0',
  /*  4155 - "wr.gpuWaitTime" */ 'w', 'r', '.', 'g', 'p', 'u', 'W', 'a', 'i', 't', 'T', 'i', 'm', 'e', '\0',
  /*  4170 - "wr.rasterizeBlobsTime" */ 'w', 'r', '.', 'r', 'a', 's', 't', 'e', 'r', 'i', 'z', 'e', 'B', 'l', 'o', 'b', 's', 'T', 'i', 'm', 'e', '\0',
  /*  4192 - "wr.rasterizeGlyphsTime" */ 'w', 'r', '.', 'r', 'a', 's', 't', 'e', 'r', 'i', 'z', 'e', 'G', 'l', 'y', 'p', 'h', 's', 'T', 'i', 'm', 'e', '\0',
  /*  4215 - "wr.rendererTime" */ 'w', 'r', '.', 'r', 'e', 'n', 'd', 'e', 'r', 'e', 'r', 'T', 'i', 'm', 'e', '\0',
  /*  4231 - "wr.rendererTimeNoSc" */ 'w', 'r', '.', 'r', 'e', 'n', 'd', 'e', 'r', 'e', 'r', 'T', 'i', 'm', 'e', 'N', 'o', 'S', 'c', '\0',
  /*  4251 - "wr.scenebuildTime" */ 'w', 'r', '.', 's', 'c', 'e', 'n', 'e', 'b', 'u', 'i', 'l', 'd', 'T', 'i', 'm', 'e', '\0',
  /*  4269 - "wr.sceneswapTime" */ 'w', 'r', '.', 's', 'c', 'e', 'n', 'e', 's', 'w', 'a', 'p', 'T', 'i', 'm', 'e', '\0',
  /*  4286 - "wr.textureCacheUpdateTime" */ 'w', 'r', '.', 't', 'e', 'x', 't', 'u', 'r', 'e', 'C', 'a', 'c', 'h', 'e', 'U', 'p', 'd', 'a', 't', 'e', 'T', 'i', 'm', 'e', '\0',
  /*  4312 - "wr.timeToFrameBuild" */ 'w', 'r', '.', 't', 'i', 'm', 'e', 'T', 'o', 'F', 'r', 'a', 'm', 'e', 'B', 'u', 'i', 'l', 'd', '\0',
  /*  4332 - "wr.timeToRenderStart" */ 'w', 'r', '.', 't', 'i', 'm', 'e', 'T', 'o', 'R', 'e', 'n', 'd', 'e', 'r', 'S', 't', 'a', 'r', 't', '\0',
  /*  4353 - "dotprint.androidDialogRequested" */ 'd', 'o', 't', 'p', 'r', 'i', 'n', 't', '.', 'a', 'n', 'd', 'r', 'o', 'i', 'd', 'D', 'i', 'a', 'l', 'o', 'g', 'R', 'e', 'q', 'u', 'e', 's', 't', 'e', 'd', '\0',
  /*  4385 - "dotprint.failure" */ 'd', 'o', 't', 'p', 'r', 'i', 'n', 't', '.', 'f', 'a', 'i', 'l', 'u', 'r', 'e', '\0',
  /*  4402 - "dotprint.requested" */ 'd', 'o', 't', 'p', 'r', 'i', 'n', 't', '.', 'r', 'e', 'q', 'u', 'e', 's', 't', 'e', 'd', '\0',
  /*  4421 - "networking.cookieAccessFixupDiff" */ 'n', 'e', 't', 'w', 'o', 'r', 'k', 'i', 'n', 'g', '.', 'c', 'o', 'o', 'k', 'i', 'e', 'A', 'c', 'c', 'e', 's', 's', 'F', 'i', 'x', 'u', 'p', 'D', 'i', 'f', 'f', '\0',
  /*  4454 - "networking.cookieCreationFixupDiff" */ 'n', 'e', 't', 'w', 'o', 'r', 'k', 'i', 'n', 'g', '.', 'c', 'o', 'o', 'k', 'i', 'e', 'C', 'r', 'e', 'a', 't', 'i', 'o', 'n', 'F', 'i', 'x', 'u', 'p', 'D', 'i', 'f', 'f', '\0',
  /*  4489 - "networking.cookieTimestampFixedCount" */ 'n', 'e', 't', 'w', 'o', 'r', 'k', 'i', 'n', 'g', '.', 'c', 'o', 'o', 'k', 'i', 'e', 'T', 'i', 'm', 'e', 's', 't', 'a', 'm', 'p', 'F', 'i', 'x', 'e', 'd', 'C', 'o', 'u', 'n', 't', '\0',
  /*  4526 - "networking.dnsFailedLookupTime" */ 'n', 'e', 't', 'w', 'o', 'r', 'k', 'i', 'n', 'g', '.', 'd', 'n', 's', 'F', 'a', 'i', 'l', 'e', 'd', 'L', 'o', 'o', 'k', 'u', 'p', 'T', 'i', 'm', 'e', '\0',
  /*  4557 - "networking.dnsLookupTime" */ 'n', 'e', 't', 'w', 'o', 'r', 'k', 'i', 'n', 'g', '.', 'd', 'n', 's', 'L', 'o', 'o', 'k', 'u', 'p', 'T', 'i', 'm', 'e', '\0',
  /*  4582 - "networking.dnsRenewalTime" */ 'n', 'e', 't', 'w', 'o', 'r', 'k', 'i', 'n', 'g', '.', 'd', 'n', 's', 'R', 'e', 'n', 'e', 'w', 'a', 'l', 'T', 'i', 'm', 'e', '\0',
  /*  4608 - "networking.dnsRenewalTimeForTtl" */ 'n', 'e', 't', 'w', 'o', 'r', 'k', 'i', 'n', 'g', '.', 'd', 'n', 's', 'R', 'e', 'n', 'e', 'w', 'a', 'l', 'T', 'i', 'm', 'e', 'F', 'o', 'r', 'T', 't', 'l', '\0',
  /*  4640 - "networking.http1DownloadThroughput" */ 'n', 'e', 't', 'w', 'o', 'r', 'k', 'i', 'n', 'g', '.', 'h', 't', 't', 'p', '1', 'D', 'o', 'w', 'n', 'l', 'o', 'a', 'd', 'T', 'h', 'r', 'o', 'u', 'g', 'h', 'p', 'u', 't', '\0',
  /*  4675 - "networking.http2DownloadThroughput" */ 'n', 'e', 't', 'w', 'o', 'r', 'k', 'i', 'n', 'g', '.', 'h', 't', 't', 'p', '2', 'D', 'o', 'w', 'n', 'l', 'o', 'a', 'd', 'T', 'h', 'r', 'o', 'u', 'g', 'h', 'p', 'u', 't', '\0',
  /*  4710 - "networking.http3DownloadThroughput" */ 'n', 'e', 't', 'w', 'o', 'r', 'k', 'i', 'n', 'g', '.', 'h', 't', 't', 'p', '3', 'D', 'o', 'w', 'n', 'l', 'o', 'a', 'd', 'T', 'h', 'r', 'o', 'u', 'g', 'h', 'p', 'u', 't', '\0',
  /*  4745 - "networking.httpContentOnstartDelay" */ 'n', 'e', 't', 'w', 'o', 'r', 'k', 'i', 'n', 'g', '.', 'h', 't', 't', 'p', 'C', 'o', 'n', 't', 'e', 'n', 't', 'O', 'n', 's', 't', 'a', 'r', 't', 'D', 'e', 'l', 'a', 'y', '\0',
  /*  4780 - "networking.httpContentOnstopDelay" */ 'n', 'e', 't', 'w', 'o', 'r', 'k', 'i', 'n', 'g', '.', 'h', 't', 't', 'p', 'C', 'o', 'n', 't', 'e', 'n', 't', 'O', 'n', 's', 't', 'o', 'p', 'D', 'e', 'l', 'a', 'y', '\0',
  /*  4814 - "networking.speculativeConnectOutcome" */ 'n', 'e', 't', 'w', 'o', 'r', 'k', 'i', 'n', 'g', '.', 's', 'p', 'e', 'c', 'u', 'l', 'a', 't', 'i', 'v', 'e', 'C', 'o', 'n', 'n', 'e', 'c', 't', 'O', 'u', 't', 'c', 'o', 'm', 'e', '\0',
  /*  4851 - "netwerk.earlyHints" */ 'n', 'e', 't', 'w', 'e', 'r', 'k', '.', 'e', 'a', 'r', 'l', 'y', 'H', 'i', 'n', 't', 's', '\0',
  /*  4870 - "netwerk.ehLinkType" */ 'n', 'e', 't', 'w', 'e', 'r', 'k', '.', 'e', 'h', 'L', 'i', 'n', 'k', 'T', 'y', 'p', 'e', '\0',
  /*  4889 - "network.corsAuthorizationHeader" */ 'n', 'e', 't', 'w', 'o', 'r', 'k', '.', 'c', 'o', 'r', 's', 'A', 'u', 't', 'h', 'o', 'r', 'i', 'z', 'a', 't', 'i', 'o', 'n', 'H', 'e', 'a', 'd', 'e', 'r', '\0',
  /*  4921 - "network.dataSizePbPerType" */ 'n', 'e', 't', 'w', 'o', 'r', 'k', '.', 'd', 'a', 't', 'a', 'S', 'i', 'z', 'e', 'P', 'b', 'P', 'e', 'r', 'T', 'y', 'p', 'e', '\0',
  /*  4947 - "network.dataSizePerType" */ 'n', 'e', 't', 'w', 'o', 'r', 'k', '.', 'd', 'a', 't', 'a', 'S', 'i', 'z', 'e', 'P', 'e', 'r', 'T', 'y', 'p', 'e', '\0',
  /*  4971 - "backgroundTasksRmdirBase.elapsedMs" */ 'b', 'a', 'c', 'k', 'g', 'r', 'o', 'u', 'n', 'd', 'T', 'a', 's', 'k', 's', 'R', 'm', 'd', 'i', 'r', 'B', 'a', 's', 'e', '.', 'e', 'l', 'a', 'p', 's', 'e', 'd', 'M', 's', '\0',
  /*  5006 - "backgroundTasksRmdirBase.metricBase" */ 'b', 'a', 'c', 'k', 'g', 'r', 'o', 'u', 'n', 'd', 'T', 'a', 's', 'k', 's', 'R', 'm', 'd', 'i', 'r', 'B', 'a', 's', 'e', '.', 'm', 'e', 't', 'r', 'i', 'c', 'B', 'a', 's', 'e', '\0',
  /*  5042 - "backgroundTasksRmdirBase.removalCount" */ 'b', 'a', 'c', 'k', 'g', 'r', 'o', 'u', 'n', 'd', 'T', 'a', 's', 'k', 's', 'R', 'm', 'd', 'i', 'r', 'B', 'a', 's', 'e', '.', 'r', 'e', 'm', 'o', 'v', 'a', 'l', 'C', 'o', 'u', 'n', 't', '\0',
  /*  5080 - "backgroundTasksRmdirBase.retryCount" */ 'b', 'a', 'c', 'k', 'g', 'r', 'o', 'u', 'n', 'd', 'T', 'a', 's', 'k', 's', 'R', 'm', 'd', 'i', 'r', 'B', 'a', 's', 'e', '.', 'r', 'e', 't', 'r', 'y', 'C', 'o', 'u', 'n', 't', '\0',
  /*  5116 - "backgroundTasksRmdirBase.succeeded" */ 'b', 'a', 'c', 'k', 'g', 'r', 'o', 'u', 'n', 'd', 'T', 'a', 's', 'k', 's', 'R', 'm', 'd', 'i', 'r', 'B', 'a', 's', 'e', '.', 's', 'u', 'c', 'c', 'e', 'e', 'd', 'e', 'd', '\0',
  /*  5151 - "backgroundTasksRmdirBase.suffixEverFailed" */ 'b', 'a', 'c', 'k', 'g', 'r', 'o', 'u', 'n', 'd', 'T', 'a', 's', 'k', 's', 'R', 'm', 'd', 'i', 'r', 'B', 'a', 's', 'e', '.', 's', 'u', 'f', 'f', 'i', 'x', 'E', 'v', 'e', 'r', 'F', 'a', 'i', 'l', 'e', 'd', '\0',
  /*  5193 - "backgroundTasksRmdirBase.suffixRemovalCount" */ 'b', 'a', 'c', 'k', 'g', 'r', 'o', 'u', 'n', 'd', 'T', 'a', 's', 'k', 's', 'R', 'm', 'd', 'i', 'r', 'B', 'a', 's', 'e', '.', 's', 'u', 'f', 'f', 'i', 'x', 'R', 'e', 'm', 'o', 'v', 'a', 'l', 'C', 'o', 'u', 'n', 't', '\0',
  /*  5237 - "backgroundTasksRmdirBase.wasFirst" */ 'b', 'a', 'c', 'k', 'g', 'r', 'o', 'u', 'n', 'd', 'T', 'a', 's', 'k', 's', 'R', 'm', 'd', 'i', 'r', 'B', 'a', 's', 'e', '.', 'w', 'a', 's', 'F', 'i', 'r', 's', 't', '\0',
  /*  5271 - "backgroundTasksRmdirHttpCache.elapsedMs" */ 'b', 'a', 'c', 'k', 'g', 'r', 'o', 'u', 'n', 'd', 'T', 'a', 's', 'k', 's', 'R', 'm', 'd', 'i', 'r', 'H', 't', 't', 'p', 'C', 'a', 'c', 'h', 'e', '.', 'e', 'l', 'a', 'p', 's', 'e', 'd', 'M', 's', '\0',
  /*  5311 - "backgroundTasksRmdirHttpCache.metricBase" */ 'b', 'a', 'c', 'k', 'g', 'r', 'o', 'u', 'n', 'd', 'T', 'a', 's', 'k', 's', 'R', 'm', 'd', 'i', 'r', 'H', 't', 't', 'p', 'C', 'a', 'c', 'h', 'e', '.', 'm', 'e', 't', 'r', 'i', 'c', 'B', 'a', 's', 'e', '\0',
  /*  5352 - "backgroundTasksRmdirHttpCache.removalCount" */ 'b', 'a', 'c', 'k', 'g', 'r', 'o', 'u', 'n', 'd', 'T', 'a', 's', 'k', 's', 'R', 'm', 'd', 'i', 'r', 'H', 't', 't', 'p', 'C', 'a', 'c', 'h', 'e', '.', 'r', 'e', 'm', 'o', 'v', 'a', 'l', 'C', 'o', 'u', 'n', 't', '\0',
  /*  5395 - "backgroundTasksRmdirHttpCache.retryCount" */ 'b', 'a', 'c', 'k', 'g', 'r', 'o', 'u', 'n', 'd', 'T', 'a', 's', 'k', 's', 'R', 'm', 'd', 'i', 'r', 'H', 't', 't', 'p', 'C', 'a', 'c', 'h', 'e', '.', 'r', 'e', 't', 'r', 'y', 'C', 'o', 'u', 'n', 't', '\0',
  /*  5436 - "backgroundTasksRmdirHttpCache.succeeded" */ 'b', 'a', 'c', 'k', 'g', 'r', 'o', 'u', 'n', 'd', 'T', 'a', 's', 'k', 's', 'R', 'm', 'd', 'i', 'r', 'H', 't', 't', 'p', 'C', 'a', 'c', 'h', 'e', '.', 's', 'u', 'c', 'c', 'e', 'e', 'd', 'e', 'd', '\0',
  /*  5476 - "backgroundTasksRmdirHttpCache.suffixEverFailed" */ 'b', 'a', 'c', 'k', 'g', 'r', 'o', 'u', 'n', 'd', 'T', 'a', 's', 'k', 's', 'R', 'm', 'd', 'i', 'r', 'H', 't', 't', 'p', 'C', 'a', 'c', 'h', 'e', '.', 's', 'u', 'f', 'f', 'i', 'x', 'E', 'v', 'e', 'r', 'F', 'a', 'i', 'l', 'e', 'd', '\0',
  /*  5523 - "backgroundTasksRmdirHttpCache.suffixRemovalCount" */ 'b', 'a', 'c', 'k', 'g', 'r', 'o', 'u', 'n', 'd', 'T', 'a', 's', 'k', 's', 'R', 'm', 'd', 'i', 'r', 'H', 't', 't', 'p', 'C', 'a', 'c', 'h', 'e', '.', 's', 'u', 'f', 'f', 'i', 'x', 'R', 'e', 'm', 'o', 'v', 'a', 'l', 'C', 'o', 'u', 'n', 't', '\0',
  /*  5572 - "backgroundTasksRmdirHttpCache.wasFirst" */ 'b', 'a', 'c', 'k', 'g', 'r', 'o', 'u', 'n', 'd', 'T', 'a', 's', 'k', 's', 'R', 'm', 'd', 'i', 'r', 'H', 't', 't', 'p', 'C', 'a', 'c', 'h', 'e', '.', 'w', 'a', 's', 'F', 'i', 'r', 's', 't', '\0',
  /*  5611 - "backgroundTasksRmdirQuota.elapsedMs" */ 'b', 'a', 'c', 'k', 'g', 'r', 'o', 'u', 'n', 'd', 'T', 'a', 's', 'k', 's', 'R', 'm', 'd', 'i', 'r', 'Q', 'u', 'o', 't', 'a', '.', 'e', 'l', 'a', 'p', 's', 'e', 'd', 'M', 's', '\0',
  /*  5647 - "backgroundTasksRmdirQuota.metricBase" */ 'b', 'a', 'c', 'k', 'g', 'r', 'o', 'u', 'n', 'd', 'T', 'a', 's', 'k', 's', 'R', 'm', 'd', 'i', 'r', 'Q', 'u', 'o', 't', 'a', '.', 'm', 'e', 't', 'r', 'i', 'c', 'B', 'a', 's', 'e', '\0',
  /*  5684 - "backgroundTasksRmdirQuota.removalCount" */ 'b', 'a', 'c', 'k', 'g', 'r', 'o', 'u', 'n', 'd', 'T', 'a', 's', 'k', 's', 'R', 'm', 'd', 'i', 'r', 'Q', 'u', 'o', 't', 'a', '.', 'r', 'e', 'm', 'o', 'v', 'a', 'l', 'C', 'o', 'u', 'n', 't', '\0',
  /*  5723 - "backgroundTasksRmdirQuota.retryCount" */ 'b', 'a', 'c', 'k', 'g', 'r', 'o', 'u', 'n', 'd', 'T', 'a', 's', 'k', 's', 'R', 'm', 'd', 'i', 'r', 'Q', 'u', 'o', 't', 'a', '.', 'r', 'e', 't', 'r', 'y', 'C', 'o', 'u', 'n', 't', '\0',
  /*  5760 - "backgroundTasksRmdirQuota.succeeded" */ 'b', 'a', 'c', 'k', 'g', 'r', 'o', 'u', 'n', 'd', 'T', 'a', 's', 'k', 's', 'R', 'm', 'd', 'i', 'r', 'Q', 'u', 'o', 't', 'a', '.', 's', 'u', 'c', 'c', 'e', 'e', 'd', 'e', 'd', '\0',
  /*  5796 - "backgroundTasksRmdirQuota.suffixEverFailed" */ 'b', 'a', 'c', 'k', 'g', 'r', 'o', 'u', 'n', 'd', 'T', 'a', 's', 'k', 's', 'R', 'm', 'd', 'i', 'r', 'Q', 'u', 'o', 't', 'a', '.', 's', 'u', 'f', 'f', 'i', 'x', 'E', 'v', 'e', 'r', 'F', 'a', 'i', 'l', 'e', 'd', '\0',
  /*  5839 - "backgroundTasksRmdirQuota.suffixRemovalCount" */ 'b', 'a', 'c', 'k', 'g', 'r', 'o', 'u', 'n', 'd', 'T', 'a', 's', 'k', 's', 'R', 'm', 'd', 'i', 'r', 'Q', 'u', 'o', 't', 'a', '.', 's', 'u', 'f', 'f', 'i', 'x', 'R', 'e', 'm', 'o', 'v', 'a', 'l', 'C', 'o', 'u', 'n', 't', '\0',
  /*  5884 - "backgroundTasksRmdirQuota.wasFirst" */ 'b', 'a', 'c', 'k', 'g', 'r', 'o', 'u', 'n', 'd', 'T', 'a', 's', 'k', 's', 'R', 'm', 'd', 'i', 'r', 'Q', 'u', 'o', 't', 'a', '.', 'w', 'a', 's', 'F', 'i', 'r', 's', 't', '\0',
  /*  5919 - "cookieBanners.normalWindowServiceMode" */ 'c', 'o', 'o', 'k', 'i', 'e', 'B', 'a', 'n', 'n', 'e', 'r', 's', '.', 'n', 'o', 'r', 'm', 'a', 'l', 'W', 'i', 'n', 'd', 'o', 'w', 'S', 'e', 'r', 'v', 'i', 'c', 'e', 'M', 'o', 'd', 'e', '\0',
  /*  5957 - "cookieBanners.privateWindowServiceMode" */ 'c', 'o', 'o', 'k', 'i', 'e', 'B', 'a', 'n', 'n', 'e', 'r', 's', '.', 'p', 'r', 'i', 'v', 'a', 't', 'e', 'W', 'i', 'n', 'd', 'o', 'w', 'S', 'e', 'r', 'v', 'i', 'c', 'e', 'M', 'o', 'd', 'e', '\0',
  /*  5996 - "cookieBanners.reload" */ 'c', 'o', 'o', 'k', 'i', 'e', 'B', 'a', 'n', 'n', 'e', 'r', 's', '.', 'r', 'e', 'l', 'o', 'a', 'd', '\0',
  /*  6017 - "cookieBanners.ruleLookupByDomain" */ 'c', 'o', 'o', 'k', 'i', 'e', 'B', 'a', 'n', 'n', 'e', 'r', 's', '.', 'r', 'u', 'l', 'e', 'L', 'o', 'o', 'k', 'u', 'p', 'B', 'y', 'D', 'o', 'm', 'a', 'i', 'n', '\0',
  /*  6050 - "cookieBanners.ruleLookupByLoad" */ 'c', 'o', 'o', 'k', 'i', 'e', 'B', 'a', 'n', 'n', 'e', 'r', 's', '.', 'r', 'u', 'l', 'e', 'L', 'o', 'o', 'k', 'u', 'p', 'B', 'y', 'L', 'o', 'a', 'd', '\0',
  /*  6081 - "cookieBanners.serviceDetectOnly" */ 'c', 'o', 'o', 'k', 'i', 'e', 'B', 'a', 'n', 'n', 'e', 'r', 's', '.', 's', 'e', 'r', 'v', 'i', 'c', 'e', 'D', 'e', 't', 'e', 'c', 't', 'O', 'n', 'l', 'y', '\0',
  /*  6113 - "cookieBannersClick.handleDuration" */ 'c', 'o', 'o', 'k', 'i', 'e', 'B', 'a', 'n', 'n', 'e', 'r', 's', 'C', 'l', 'i', 'c', 'k', '.', 'h', 'a', 'n', 'd', 'l', 'e', 'D', 'u', 'r', 'a', 't', 'i', 'o', 'n', '\0',
  /*  6147 - "cookieBannersClick.result" */ 'c', 'o', 'o', 'k', 'i', 'e', 'B', 'a', 'n', 'n', 'e', 'r', 's', 'C', 'l', 'i', 'c', 'k', '.', 'r', 'e', 's', 'u', 'l', 't', '\0',
  /*  6173 - "crash.processType" */ 'c', 'r', 'a', 's', 'h', '.', 'p', 'r', 'o', 'c', 'e', 's', 's', 'T', 'y', 'p', 'e', '\0',
  /*  6191 - "crash.startup" */ 'c', 'r', 'a', 's', 'h', '.', 's', 't', 'a', 'r', 't', 'u', 'p', '\0',
  /*  6205 - "crash.time" */ 'c', 'r', 'a', 's', 'h', '.', 't', 'i', 'm', 'e', '\0',
  /*  6216 - "crash.uptime" */ 'c', 'r', 'a', 's', 'h', '.', 'u', 'p', 't', 'i', 'm', 'e', '\0',
  /*  6229 - "extensions.processEvent" */ 'e', 'x', 't', 'e', 'n', 's', 'i', 'o', 'n', 's', '.', 'p', 'r', 'o', 'c', 'e', 's', 's', 'E', 'v', 'e', 'n', 't', '\0',
  /*  6253 - "extensions.startupCacheLoadTime" */ 'e', 'x', 't', 'e', 'n', 's', 'i', 'o', 'n', 's', '.', 's', 't', 'a', 'r', 't', 'u', 'p', 'C', 'a', 'c', 'h', 'e', 'L', 'o', 'a', 'd', 'T', 'i', 'm', 'e', '\0',
  /*  6285 - "extensions.useRemotePolicy" */ 'e', 'x', 't', 'e', 'n', 's', 'i', 'o', 'n', 's', '.', 'u', 's', 'e', 'R', 'e', 'm', 'o', 't', 'e', 'P', 'o', 'l', 'i', 'c', 'y', '\0',
  /*  6312 - "extensions.useRemotePref" */ 'e', 'x', 't', 'e', 'n', 's', 'i', 'o', 'n', 's', '.', 'u', 's', 'e', 'R', 'e', 'm', 'o', 't', 'e', 'P', 'r', 'e', 'f', '\0',
  /*  6337 - "extensionsApisDnr.evaluateRulesCountMax" */ 'e', 'x', 't', 'e', 'n', 's', 'i', 'o', 'n', 's', 'A', 'p', 'i', 's', 'D', 'n', 'r', '.', 'e', 'v', 'a', 'l', 'u', 'a', 't', 'e', 'R', 'u', 'l', 'e', 's', 'C', 'o', 'u', 'n', 't', 'M', 'a', 'x', '\0',
  /*  6377 - "extensionsApisDnr.evaluateRulesTime" */ 'e', 'x', 't', 'e', 'n', 's', 'i', 'o', 'n', 's', 'A', 'p', 'i', 's', 'D', 'n', 'r', '.', 'e', 'v', 'a', 'l', 'u', 'a', 't', 'e', 'R', 'u', 'l', 'e', 's', 'T', 'i', 'm', 'e', '\0',
  /*  6413 - "extensionsApisDnr.startupCacheEntries" */ 'e', 'x', 't', 'e', 'n', 's', 'i', 'o', 'n', 's', 'A', 'p', 'i', 's', 'D', 'n', 'r', '.', 's', 't', 'a', 'r', 't', 'u', 'p', 'C', 'a', 'c', 'h', 'e', 'E', 'n', 't', 'r', 'i', 'e', 's', '\0',
  /*  6451 - "extensionsApisDnr.startupCacheReadSize" */ 'e', 'x', 't', 'e', 'n', 's', 'i', 'o', 'n', 's', 'A', 'p', 'i', 's', 'D', 'n', 'r', '.', 's', 't', 'a', 'r', 't', 'u', 'p', 'C', 'a', 'c', 'h', 'e', 'R', 'e', 'a', 'd', 'S', 'i', 'z', 'e', '\0',
  /*  6490 - "extensionsApisDnr.startupCacheReadTime" */ 'e', 'x', 't', 'e', 'n', 's', 'i', 'o', 'n', 's', 'A', 'p', 'i', 's', 'D', 'n', 'r', '.', 's', 't', 'a', 'r', 't', 'u', 'p', 'C', 'a', 'c', 'h', 'e', 'R', 'e', 'a', 'd', 'T', 'i', 'm', 'e', '\0',
  /*  6529 - "extensionsApisDnr.startupCacheWriteSize" */ 'e', 'x', 't', 'e', 'n', 's', 'i', 'o', 'n', 's', 'A', 'p', 'i', 's', 'D', 'n', 'r', '.', 's', 't', 'a', 'r', 't', 'u', 'p', 'C', 'a', 'c', 'h', 'e', 'W', 'r', 'i', 't', 'e', 'S', 'i', 'z', 'e', '\0',
  /*  6569 - "extensionsApisDnr.startupCacheWriteTime" */ 'e', 'x', 't', 'e', 'n', 's', 'i', 'o', 'n', 's', 'A', 'p', 'i', 's', 'D', 'n', 'r', '.', 's', 't', 'a', 'r', 't', 'u', 'p', 'C', 'a', 'c', 'h', 'e', 'W', 'r', 'i', 't', 'e', 'T', 'i', 'm', 'e', '\0',
  /*  6609 - "extensionsApisDnr.validateRulesTime" */ 'e', 'x', 't', 'e', 'n', 's', 'i', 'o', 'n', 's', 'A', 'p', 'i', 's', 'D', 'n', 'r', '.', 'v', 'a', 'l', 'i', 'd', 'a', 't', 'e', 'R', 'u', 'l', 'e', 's', 'T', 'i', 'm', 'e', '\0',
  /*  6645 - "extensionsQuarantinedDomains.listhash" */ 'e', 'x', 't', 'e', 'n', 's', 'i', 'o', 'n', 's', 'Q', 'u', 'a', 'r', 'a', 'n', 't', 'i', 'n', 'e', 'd', 'D', 'o', 'm', 'a', 'i', 'n', 's', '.', 'l', 'i', 's', 't', 'h', 'a', 's', 'h', '\0',
  /*  6683 - "extensionsQuarantinedDomains.listsize" */ 'e', 'x', 't', 'e', 'n', 's', 'i', 'o', 'n', 's', 'Q', 'u', 'a', 'r', 'a', 'n', 't', 'i', 'n', 'e', 'd', 'D', 'o', 'm', 'a', 'i', 'n', 's', '.', 'l', 'i', 's', 't', 's', 'i', 'z', 'e', '\0',
  /*  6721 - "extensionsQuarantinedDomains.remotehash" */ 'e', 'x', 't', 'e', 'n', 's', 'i', 'o', 'n', 's', 'Q', 'u', 'a', 'r', 'a', 'n', 't', 'i', 'n', 'e', 'd', 'D', 'o', 'm', 'a', 'i', 'n', 's', '.', 'r', 'e', 'm', 'o', 't', 'e', 'h', 'a', 's', 'h', '\0',
  /*  6761 - "formautofillCreditcards.autofillProfilesCount" */ 'f', 'o', 'r', 'm', 'a', 'u', 't', 'o', 'f', 'i', 'l', 'l', 'C', 'r', 'e', 'd', 'i', 't', 'c', 'a', 'r', 'd', 's', '.', 'a', 'u', 't', 'o', 'f', 'i', 'l', 'l', 'P', 'r', 'o', 'f', 'i', 'l', 'e', 's', 'C', 'o', 'u', 'n', 't', '\0',
  /*  6807 - "formautofillCreditcards.formCleared" */ 'f', 'o', 'r', 'm', 'a', 'u', 't', 'o', 'f', 'i', 'l', 'l', 'C', 'r', 'e', 'd', 'i', 't', 'c', 'a', 'r', 'd', 's', '.', 'f', 'o', 'r', 'm', 'C', 'l', 'e', 'a', 'r', 'e', 'd', '\0',
  /*  6843 - "formautofillCreditcards.formDetected" */ 'f', 'o', 'r', 'm', 'a', 'u', 't', 'o', 'f', 'i', 'l', 'l', 'C', 'r', 'e', 'd', 'i', 't', 'c', 'a', 'r', 'd', 's', '.', 'f', 'o', 'r', 'm', 'D', 'e', 't', 'e', 'c', 't', 'e', 'd', '\0',
  /*  6880 - "formautofillCreditcards.formFilled" */ 'f', 'o', 'r', 'm', 'a', 'u', 't', 'o', 'f', 'i', 'l', 'l', 'C', 'r', 'e', 'd', 'i', 't', 'c', 'a', 'r', 'd', 's', '.', 'f', 'o', 'r', 'm', 'F', 'i', 'l', 'l', 'e', 'd', '\0',
  /*  6915 - "formautofillCreditcards.formFilledModified" */ 'f', 'o', 'r', 'm', 'a', 'u', 't', 'o', 'f', 'i', 'l', 'l', 'C', 'r', 'e', 'd', 'i', 't', 'c', 'a', 'r', 'd', 's', '.', 'f', 'o', 'r', 'm', 'F', 'i', 'l', 'l', 'e', 'd', 'M', 'o', 'd', 'i', 'f', 'i', 'e', 'd', '\0',
  /*  6958 - "formautofillCreditcards.formPopupShown" */ 'f', 'o', 'r', 'm', 'a', 'u', 't', 'o', 'f', 'i', 'l', 'l', 'C', 'r', 'e', 'd', 'i', 't', 'c', 'a', 'r', 'd', 's', '.', 'f', 'o', 'r', 'm', 'P', 'o', 'p', 'u', 'p', 'S', 'h', 'o', 'w', 'n', '\0',
  /*  6997 - "formautofillCreditcards.formSubmitted" */ 'f', 'o', 'r', 'm', 'a', 'u', 't', 'o', 'f', 'i', 'l', 'l', 'C', 'r', 'e', 'd', 'i', 't', 'c', 'a', 'r', 'd', 's', '.', 'f', 'o', 'r', 'm', 'S', 'u', 'b', 'm', 'i', 't', 't', 'e', 'd', '\0',
  /*  7035 - "fog.failedIdleRegistration" */ 'f', 'o', 'g', '.', 'f', 'a', 'i', 'l', 'e', 'd', 'I', 'd', 'l', 'e', 'R', 'e', 'g', 'i', 's', 't', 'r', 'a', 't', 'i', 'o', 'n', '\0',
  /*  7062 - "fog.initialization" */ 'f', 'o', 'g', '.', 'i', 'n', 'i', 't', 'i', 'a', 'l', 'i', 'z', 'a', 't', 'i', 'o', 'n', '\0',
  /*  7081 - "fog.initsDuringShutdown" */ 'f', 'o', 'g', '.', 'i', 'n', 'i', 't', 's', 'D', 'u', 'r', 'i', 'n', 'g', 'S', 'h', 'u', 't', 'd', 'o', 'w', 'n', '\0',
  /*  7105 - "fog.maxPingsPerMinute" */ 'f', 'o', 'g', '.', 'm', 'a', 'x', 'P', 'i', 'n', 'g', 's', 'P', 'e', 'r', 'M', 'i', 'n', 'u', 't', 'e', '\0',
  /*  7127 - "fogIpc.bufferSizes" */ 'f', 'o', 'g', 'I', 'p', 'c', '.', 'b', 'u', 'f', 'f', 'e', 'r', 'S', 'i', 'z', 'e', 's', '\0',
  /*  7146 - "fogIpc.flushDurations" */ 'f', 'o', 'g', 'I', 'p', 'c', '.', 'f', 'l', 'u', 's', 'h', 'D', 'u', 'r', 'a', 't', 'i', 'o', 'n', 's', '\0',
  /*  7168 - "fogIpc.flushFailures" */ 'f', 'o', 'g', 'I', 'p', 'c', '.', 'f', 'l', 'u', 's', 'h', 'F', 'a', 'i', 'l', 'u', 'r', 'e', 's', '\0',
  /*  7189 - "fogIpc.replayFailures" */ 'f', 'o', 'g', 'I', 'p', 'c', '.', 'r', 'e', 'p', 'l', 'a', 'y', 'F', 'a', 'i', 'l', 'u', 'r', 'e', 's', '\0',
  /*  7211 - "fogIpc.shutdownRegistrationFailures" */ 'f', 'o', 'g', 'I', 'p', 'c', '.', 's', 'h', 'u', 't', 'd', 'o', 'w', 'n', 'R', 'e', 'g', 'i', 's', 't', 'r', 'a', 't', 'i', 'o', 'n', 'F', 'a', 'i', 'l', 'u', 'r', 'e', 's', '\0',
  /*  7247 - "fogValidation.validateEarlyEvent" */ 'f', 'o', 'g', 'V', 'a', 'l', 'i', 'd', 'a', 't', 'i', 'o', 'n', '.', 'v', 'a', 'l', 'i', 'd', 'a', 't', 'e', 'E', 'a', 'r', 'l', 'y', 'E', 'v', 'e', 'n', 't', '\0',
  /*  7280 - "testOnly.badCode" */ 't', 'e', 's', 't', 'O', 'n', 'l', 'y', '.', 'b', 'a', 'd', 'C', 'o', 'd', 'e', '\0',
  /*  7297 - "testOnly.canWeFlagIt" */ 't', 'e', 's', 't', 'O', 'n', 'l', 'y', '.', 'c', 'a', 'n', 'W', 'e', 'F', 'l', 'a', 'g', 'I', 't', '\0',
  /*  7318 - "testOnly.canWeTimeIt" */ 't', 'e', 's', 't', 'O', 'n', 'l', 'y', '.', 'c', 'a', 'n', 'W', 'e', 'T', 'i', 'm', 'e', 'I', 't', '\0',
  /*  7339 - "testOnly.cheesyString" */ 't', 'e', 's', 't', 'O', 'n', 'l', 'y', '.', 'c', 'h', 'e', 'e', 's', 'y', 'S', 't', 'r', 'i', 'n', 'g', '\0',
  /*  7361 - "testOnly.cheesyStringList" */ 't', 'e', 's', 't', 'O', 'n', 'l', 'y', '.', 'c', 'h', 'e', 'e', 's', 'y', 'S', 't', 'r', 'i', 'n', 'g', 'L', 'i', 's', 't', '\0',
  /*  7387 - "testOnly.doYouRemember" */ 't', 'e', 's', 't', 'O', 'n', 'l', 'y', '.', 'd', 'o', 'Y', 'o', 'u', 'R', 'e', 'm', 'e', 'm', 'b', 'e', 'r', '\0',
  /*  7410 - "testOnly.mabelsBalloonLabels" */ 't', 'e', 's', 't', 'O', 'n', 'l', 'y', '.', 'm', 'a', 'b', 'e', 'l', 's', 'B', 'a', 'l', 'l', 'o', 'o', 'n', 'L', 'a', 'b', 'e', 'l', 's', '\0',
  /*  7439 - "testOnly.mabelsBalloonStrings" */ 't', 'e', 's', 't', 'O', 'n', 'l', 'y', '.', 'm', 'a', 'b', 'e', 'l', 's', 'B', 'a', 'l', 'l', 'o', 'o', 'n', 'S', 't', 'r', 'i', 'n', 'g', 's', '\0',
  /*  7469 - "testOnly.mabelsBathroomCounters" */ 't', 'e', 's', 't', 'O', 'n', 'l', 'y', '.', 'm', 'a', 'b', 'e', 'l', 's', 'B', 'a', 't', 'h', 'r', 'o', 'o', 'm', 'C', 'o', 'u', 'n', 't', 'e', 'r', 's', '\0',
  /*  7501 - "testOnly.mabelsKitchenCounters" */ 't', 'e', 's', 't', 'O', 'n', 'l', 'y', '.', 'm', 'a', 'b', 'e', 'l', 's', 'K', 'i', 't', 'c', 'h', 'e', 'n', 'C', 'o', 'u', 'n', 't', 'e', 'r', 's', '\0',
  /*  7532 - "testOnly.mabelsLabelMaker" */ 't', 'e', 's', 't', 'O', 'n', 'l', 'y', '.', 'm', 'a', 'b', 'e', 'l', 's', 'L', 'a', 'b', 'e', 'l', 'M', 'a', 'k', 'e', 'r', '\0',
  /*  7558 - "testOnly.mabelsLabeledCounters" */ 't', 'e', 's', 't', 'O', 'n', 'l', 'y', '.', 'm', 'a', 'b', 'e', 'l', 's', 'L', 'a', 'b', 'e', 'l', 'e', 'd', 'C', 'o', 'u', 'n', 't', 'e', 'r', 's', '\0',
  /*  7589 - "testOnly.mabelsLikeBalloons" */ 't', 'e', 's', 't', 'O', 'n', 'l', 'y', '.', 'm', 'a', 'b', 'e', 'l', 's', 'L', 'i', 'k', 'e', 'B', 'a', 'l', 'l', 'o', 'o', 'n', 's', '\0',
  /*  7617 - "testOnly.mabelsLikeLabeledBalloons" */ 't', 'e', 's', 't', 'O', 'n', 'l', 'y', '.', 'm', 'a', 'b', 'e', 'l', 's', 'L', 'i', 'k', 'e', 'L', 'a', 'b', 'e', 'l', 'e', 'd', 'B', 'a', 'l', 'l', 'o', 'o', 'n', 's', '\0',
  /*  7652 - "testOnly.meaningOfLife" */ 't', 'e', 's', 't', 'O', 'n', 'l', 'y', '.', 'm', 'e', 'a', 'n', 'i', 'n', 'g', 'O', 'f', 'L', 'i', 'f', 'e', '\0',
  /*  7675 - "testOnly.mirrorTime" */ 't', 'e', 's', 't', 'O', 'n', 'l', 'y', '.', 'm', 'i', 'r', 'r', 'o', 'r', 'T', 'i', 'm', 'e', '\0',
  /*  7695 - "testOnly.mirrorTimeNanos" */ 't', 'e', 's', 't', 'O', 'n', 'l', 'y', '.', 'm', 'i', 'r', 'r', 'o', 'r', 'T', 'i', 'm', 'e', 'N', 'a', 'n', 'o', 's', '\0',
  /*  7720 - "testOnly.mirrorsForLabeledBools" */ 't', 'e', 's', 't', 'O', 'n', 'l', 'y', '.', 'm', 'i', 'r', 'r', 'o', 'r', 's', 'F', 'o', 'r', 'L', 'a', 'b', 'e', 'l', 'e', 'd', 'B', 'o', 'o', 'l', 's', '\0',
  /*  7752 - "testOnly.onePingOneBool" */ 't', 'e', 's', 't', 'O', 'n', 'l', 'y', '.', 'o', 'n', 'e', 'P', 'i', 'n', 'g', 'O', 'n', 'e', 'B', 'o', 'o', 'l', '\0',
  /*  7776 - "testOnly.whatADate" */ 't', 'e', 's', 't', 'O', 'n', 'l', 'y', '.', 'w', 'h', 'a', 't', 'A', 'D', 'a', 't', 'e', '\0',
  /*  7795 - "testOnly.whatIdIt" */ 't', 'e', 's', 't', 'O', 'n', 'l', 'y', '.', 'w', 'h', 'a', 't', 'I', 'd', 'I', 't', '\0',
  /*  7813 - "testOnly.whatTimeIsIt" */ 't', 'e', 's', 't', 'O', 'n', 'l', 'y', '.', 'w', 'h', 'a', 't', 'T', 'i', 'm', 'e', 'I', 's', 'I', 't', '\0',
  /*  7835 - "testOnlyIpc.aBool" */ 't', 'e', 's', 't', 'O', 'n', 'l', 'y', 'I', 'p', 'c', '.', 'a', 'B', 'o', 'o', 'l', '\0',
  /*  7853 - "testOnlyIpc.aCounter" */ 't', 'e', 's', 't', 'O', 'n', 'l', 'y', 'I', 'p', 'c', '.', 'a', 'C', 'o', 'u', 'n', 't', 'e', 'r', '\0',
  /*  7874 - "testOnlyIpc.aCustomDist" */ 't', 'e', 's', 't', 'O', 'n', 'l', 'y', 'I', 'p', 'c', '.', 'a', 'C', 'u', 's', 't', 'o', 'm', 'D', 'i', 's', 't', '\0',
  /*  7898 - "testOnlyIpc.aDate" */ 't', 'e', 's', 't', 'O', 'n', 'l', 'y', 'I', 'p', 'c', '.', 'a', 'D', 'a', 't', 'e', '\0',
  /*  7916 - "testOnlyIpc.aLabeledCounter" */ 't', 'e', 's', 't', 'O', 'n', 'l', 'y', 'I', 'p', 'c', '.', 'a', 'L', 'a', 'b', 'e', 'l', 'e', 'd', 'C', 'o', 'u', 'n', 't', 'e', 'r', '\0',
  /*  7944 - "testOnlyIpc.aMemoryDist" */ 't', 'e', 's', 't', 'O', 'n', 'l', 'y', 'I', 'p', 'c', '.', 'a', 'M', 'e', 'm', 'o', 'r', 'y', 'D', 'i', 's', 't', '\0',
  /*  7968 - "testOnlyIpc.aQuantity" */ 't', 'e', 's', 't', 'O', 'n', 'l', 'y', 'I', 'p', 'c', '.', 'a', 'Q', 'u', 'a', 'n', 't', 'i', 't', 'y', '\0',
  /*  7990 - "testOnlyIpc.aString" */ 't', 'e', 's', 't', 'O', 'n', 'l', 'y', 'I', 'p', 'c', '.', 'a', 'S', 't', 'r', 'i', 'n', 'g', '\0',
  /*  8010 - "testOnlyIpc.aStringList" */ 't', 'e', 's', 't', 'O', 'n', 'l', 'y', 'I', 'p', 'c', '.', 'a', 'S', 't', 'r', 'i', 'n', 'g', 'L', 'i', 's', 't', '\0',
  /*  8034 - "testOnlyIpc.aText" */ 't', 'e', 's', 't', 'O', 'n', 'l', 'y', 'I', 'p', 'c', '.', 'a', 'T', 'e', 'x', 't', '\0',
  /*  8052 - "testOnlyIpc.aTimingDist" */ 't', 'e', 's', 't', 'O', 'n', 'l', 'y', 'I', 'p', 'c', '.', 'a', 'T', 'i', 'm', 'i', 'n', 'g', 'D', 'i', 's', 't', '\0',
  /*  8076 - "testOnlyIpc.aUrl" */ 't', 'e', 's', 't', 'O', 'n', 'l', 'y', 'I', 'p', 'c', '.', 'a', 'U', 'r', 'l', '\0',
  /*  8093 - "testOnlyIpc.aUuid" */ 't', 'e', 's', 't', 'O', 'n', 'l', 'y', 'I', 'p', 'c', '.', 'a', 'U', 'u', 'i', 'd', '\0',
  /*  8111 - "testOnlyIpc.anEvent" */ 't', 'e', 's', 't', 'O', 'n', 'l', 'y', 'I', 'p', 'c', '.', 'a', 'n', 'E', 'v', 'e', 'n', 't', '\0',
  /*  8131 - "testOnlyIpc.anExternalDenominator" */ 't', 'e', 's', 't', 'O', 'n', 'l', 'y', 'I', 'p', 'c', '.', 'a', 'n', 'E', 'x', 't', 'e', 'r', 'n', 'a', 'l', 'D', 'e', 'n', 'o', 'm', 'i', 'n', 'a', 't', 'o', 'r', '\0',
  /*  8165 - "testOnlyIpc.anotherLabeledCounter" */ 't', 'e', 's', 't', 'O', 'n', 'l', 'y', 'I', 'p', 'c', '.', 'a', 'n', 'o', 't', 'h', 'e', 'r', 'L', 'a', 'b', 'e', 'l', 'e', 'd', 'C', 'o', 'u', 'n', 't', 'e', 'r', '\0',
  /*  8199 - "testOnlyIpc.eventWithExtra" */ 't', 'e', 's', 't', 'O', 'n', 'l', 'y', 'I', 'p', 'c', '.', 'e', 'v', 'e', 'n', 't', 'W', 'i', 't', 'h', 'E', 'x', 't', 'r', 'a', '\0',
  /*  8226 - "testOnlyIpc.irate" */ 't', 'e', 's', 't', 'O', 'n', 'l', 'y', 'I', 'p', 'c', '.', 'i', 'r', 'a', 't', 'e', '\0',
  /*  8244 - "testOnlyIpc.noExtraEvent" */ 't', 'e', 's', 't', 'O', 'n', 'l', 'y', 'I', 'p', 'c', '.', 'n', 'o', 'E', 'x', 't', 'r', 'a', 'E', 'v', 'e', 'n', 't', '\0',
  /*  8269 - "testOnlyIpc.rateWithExternalDenominator" */ 't', 'e', 's', 't', 'O', 'n', 'l', 'y', 'I', 'p', 'c', '.', 'r', 'a', 't', 'e', 'W', 'i', 't', 'h', 'E', 'x', 't', 'e', 'r', 'n', 'a', 'l', 'D', 'e', 'n', 'o', 'm', 'i', 'n', 'a', 't', 'o', 'r', '\0',
  /*  8309 - "testOnlyJog.aCounter" */ 't', 'e', 's', 't', 'O', 'n', 'l', 'y', 'J', 'o', 'g', '.', 'a', 'C', 'o', 'u', 'n', 't', 'e', 'r', '\0',
  /*  8330 - "testOnlyJog.anEvent" */ 't', 'e', 's', 't', 'O', 'n', 'l', 'y', 'J', 'o', 'g', '.', 'a', 'n', 'E', 'v', 'e', 'n', 't', '\0',
  /*  8350 - "nimbusEvents.enrollFailed" */ 'n', 'i', 'm', 'b', 'u', 's', 'E', 'v', 'e', 'n', 't', 's', '.', 'e', 'n', 'r', 'o', 'l', 'l', 'F', 'a', 'i', 'l', 'e', 'd', '\0',
  /*  8376 - "nimbusEvents.enrollment" */ 'n', 'i', 'm', 'b', 'u', 's', 'E', 'v', 'e', 'n', 't', 's', '.', 'e', 'n', 'r', 'o', 'l', 'l', 'm', 'e', 'n', 't', '\0',
  /*  8400 - "nimbusEvents.exposure" */ 'n', 'i', 'm', 'b', 'u', 's', 'E', 'v', 'e', 'n', 't', 's', '.', 'e', 'x', 'p', 'o', 's', 'u', 'r', 'e', '\0',
  /*  8422 - "nimbusEvents.unenrollFailed" */ 'n', 'i', 'm', 'b', 'u', 's', 'E', 'v', 'e', 'n', 't', 's', '.', 'u', 'n', 'e', 'n', 'r', 'o', 'l', 'l', 'F', 'a', 'i', 'l', 'e', 'd', '\0',
  /*  8450 - "nimbusEvents.unenrollment" */ 'n', 'i', 'm', 'b', 'u', 's', 'E', 'v', 'e', 'n', 't', 's', '.', 'u', 'n', 'e', 'n', 'r', 'o', 'l', 'l', 'm', 'e', 'n', 't', '\0',
  /*  8476 - "nimbusEvents.validationFailed" */ 'n', 'i', 'm', 'b', 'u', 's', 'E', 'v', 'e', 'n', 't', 's', '.', 'v', 'a', 'l', 'i', 'd', 'a', 't', 'i', 'o', 'n', 'F', 'a', 'i', 'l', 'e', 'd', '\0',
  /*  8506 - "pwmgr.formAutofillResult" */ 'p', 'w', 'm', 'g', 'r', '.', 'f', 'o', 'r', 'm', 'A', 'u', 't', 'o', 'f', 'i', 'l', 'l', 'R', 'e', 's', 'u', 'l', 't', '\0',
  /*  8531 - "pdfjs.buttons" */ 'p', 'd', 'f', 'j', 's', '.', 'b', 'u', 't', 't', 'o', 'n', 's', '\0',
  /*  8545 - "pdfjs.editing" */ 'p', 'd', 'f', 'j', 's', '.', 'e', 'd', 'i', 't', 'i', 'n', 'g', '\0',
  /*  8559 - "pdfjs.geckoview" */ 'p', 'd', 'f', 'j', 's', '.', 'g', 'e', 'c', 'k', 'o', 'v', 'i', 'e', 'w', '\0',
  /*  8575 - "pdfjs.timeToView" */ 'p', 'd', 'f', 'j', 's', '.', 't', 'i', 'm', 'e', 'T', 'o', 'V', 'i', 'e', 'w', '\0',
  /*  8592 - "pdfjs.used" */ 'p', 'd', 'f', 'j', 's', '.', 'u', 's', 'e', 'd', '\0',
  /*  8603 - "power.cpuTimeBogusValues" */ 'p', 'o', 'w', 'e', 'r', '.', 'c', 'p', 'u', 'T', 'i', 'm', 'e', 'B', 'o', 'g', 'u', 's', 'V', 'a', 'l', 'u', 'e', 's', '\0',
  /*  8628 - "power.cpuTimePerProcessTypeMs" */ 'p', 'o', 'w', 'e', 'r', '.', 'c', 'p', 'u', 'T', 'i', 'm', 'e', 'P', 'e', 'r', 'P', 'r', 'o', 'c', 'e', 's', 's', 'T', 'y', 'p', 'e', 'M', 's', '\0',
  /*  8658 - "power.cpuTimePerTrackerTypeMs" */ 'p', 'o', 'w', 'e', 'r', '.', 'c', 'p', 'u', 'T', 'i', 'm', 'e', 'P', 'e', 'r', 'T', 'r', 'a', 'c', 'k', 'e', 'r', 'T', 'y', 'p', 'e', 'M', 's', '\0',
  /*  8688 - "power.gpuTimeBogusValues" */ 'p', 'o', 'w', 'e', 'r', '.', 'g', 'p', 'u', 'T', 'i', 'm', 'e', 'B', 'o', 'g', 'u', 's', 'V', 'a', 'l', 'u', 'e', 's', '\0',
  /*  8713 - "power.gpuTimePerProcessTypeMs" */ 'p', 'o', 'w', 'e', 'r', '.', 'g', 'p', 'u', 'T', 'i', 'm', 'e', 'P', 'e', 'r', 'P', 'r', 'o', 'c', 'e', 's', 's', 'T', 'y', 'p', 'e', 'M', 's', '\0',
  /*  8743 - "power.totalCpuTimeMs" */ 'p', 'o', 'w', 'e', 'r', '.', 't', 'o', 't', 'a', 'l', 'C', 'p', 'u', 'T', 'i', 'm', 'e', 'M', 's', '\0',
  /*  8764 - "power.totalGpuTimeMs" */ 'p', 'o', 'w', 'e', 'r', '.', 't', 'o', 't', 'a', 'l', 'G', 'p', 'u', 'T', 'i', 'm', 'e', 'M', 's', '\0',
  /*  8785 - "power.totalThreadWakeups" */ 'p', 'o', 'w', 'e', 'r', '.', 't', 'o', 't', 'a', 'l', 'T', 'h', 'r', 'e', 'a', 'd', 'W', 'a', 'k', 'e', 'u', 'p', 's', '\0',
  /*  8810 - "power.wakeupsPerProcessType" */ 'p', 'o', 'w', 'e', 'r', '.', 'w', 'a', 'k', 'e', 'u', 'p', 's', 'P', 'e', 'r', 'P', 'r', 'o', 'c', 'e', 's', 's', 'T', 'y', 'p', 'e', '\0',
  /*  8838 - "powerBattery.percentageWhenUserActive" */ 'p', 'o', 'w', 'e', 'r', 'B', 'a', 't', 't', 'e', 'r', 'y', '.', 'p', 'e', 'r', 'c', 'e', 'n', 't', 'a', 'g', 'e', 'W', 'h', 'e', 'n', 'U', 's', 'e', 'r', 'A', 'c', 't', 'i', 'v', 'e', '\0',
  /*  8876 - "powerCpuMsPerThread.contentBackground" */ 'p', 'o', 'w', 'e', 'r', 'C', 'p', 'u', 'M', 's', 'P', 'e', 'r', 'T', 'h', 'r', 'e', 'a', 'd', '.', 'c', 'o', 'n', 't', 'e', 'n', 't', 'B', 'a', 'c', 'k', 'g', 'r', 'o', 'u', 'n', 'd', '\0',
  /*  8914 - "powerCpuMsPerThread.contentForeground" */ 'p', 'o', 'w', 'e', 'r', 'C', 'p', 'u', 'M', 's', 'P', 'e', 'r', 'T', 'h', 'r', 'e', 'a', 'd', '.', 'c', 'o', 'n', 't', 'e', 'n', 't', 'F', 'o', 'r', 'e', 'g', 'r', 'o', 'u', 'n', 'd', '\0',
  /*  8952 - "powerCpuMsPerThread.gpuProcess" */ 'p', 'o', 'w', 'e', 'r', 'C', 'p', 'u', 'M', 's', 'P', 'e', 'r', 'T', 'h', 'r', 'e', 'a', 'd', '.', 'g', 'p', 'u', 'P', 'r', 'o', 'c', 'e', 's', 's', '\0',
  /*  8983 - "powerCpuMsPerThread.parentActive" */ 'p', 'o', 'w', 'e', 'r', 'C', 'p', 'u', 'M', 's', 'P', 'e', 'r', 'T', 'h', 'r', 'e', 'a', 'd', '.', 'p', 'a', 'r', 'e', 'n', 't', 'A', 'c', 't', 'i', 'v', 'e', '\0',
  /*  9016 - "powerCpuMsPerThread.parentInactive" */ 'p', 'o', 'w', 'e', 'r', 'C', 'p', 'u', 'M', 's', 'P', 'e', 'r', 'T', 'h', 'r', 'e', 'a', 'd', '.', 'p', 'a', 'r', 'e', 'n', 't', 'I', 'n', 'a', 'c', 't', 'i', 'v', 'e', '\0',
  /*  9051 - "powerWakeupsPerThread.contentBackground" */ 'p', 'o', 'w', 'e', 'r', 'W', 'a', 'k', 'e', 'u', 'p', 's', 'P', 'e', 'r', 'T', 'h', 'r', 'e', 'a', 'd', '.', 'c', 'o', 'n', 't', 'e', 'n', 't', 'B', 'a', 'c', 'k', 'g', 'r', 'o', 'u', 'n', 'd', '\0',
  /*  9091 - "powerWakeupsPerThread.contentForeground" */ 'p', 'o', 'w', 'e', 'r', 'W', 'a', 'k', 'e', 'u', 'p', 's', 'P', 'e', 'r', 'T', 'h', 'r', 'e', 'a', 'd', '.', 'c', 'o', 'n', 't', 'e', 'n', 't', 'F', 'o', 'r', 'e', 'g', 'r', 'o', 'u', 'n', 'd', '\0',
  /*  9131 - "powerWakeupsPerThread.gpuProcess" */ 'p', 'o', 'w', 'e', 'r', 'W', 'a', 'k', 'e', 'u', 'p', 's', 'P', 'e', 'r', 'T', 'h', 'r', 'e', 'a', 'd', '.', 'g', 'p', 'u', 'P', 'r', 'o', 'c', 'e', 's', 's', '\0',
  /*  9164 - "powerWakeupsPerThread.parentActive" */ 'p', 'o', 'w', 'e', 'r', 'W', 'a', 'k', 'e', 'u', 'p', 's', 'P', 'e', 'r', 'T', 'h', 'r', 'e', 'a', 'd', '.', 'p', 'a', 'r', 'e', 'n', 't', 'A', 'c', 't', 'i', 'v', 'e', '\0',
  /*  9199 - "powerWakeupsPerThread.parentInactive" */ 'p', 'o', 'w', 'e', 'r', 'W', 'a', 'k', 'e', 'u', 'p', 's', 'P', 'e', 'r', 'T', 'h', 'r', 'e', 'a', 'd', '.', 'p', 'a', 'r', 'e', 'n', 't', 'I', 'n', 'a', 'c', 't', 'i', 'v', 'e', '\0',
  /*  9236 - "fingerprintingProtection.canvasNoiseCalculateTime" */ 'f', 'i', 'n', 'g', 'e', 'r', 'p', 'r', 'i', 'n', 't', 'i', 'n', 'g', 'P', 'r', 'o', 't', 'e', 'c', 't', 'i', 'o', 'n', '.', 'c', 'a', 'n', 'v', 'a', 's', 'N', 'o', 'i', 's', 'e', 'C', 'a', 'l', 'c', 'u', 'l', 'a', 't', 'e', 'T', 'i', 'm', 'e', '\0',
  /*  9286 - "searchEngineDefault.changed" */ 's', 'e', 'a', 'r', 'c', 'h', 'E', 'n', 'g', 'i', 'n', 'e', 'D', 'e', 'f', 'a', 'u', 'l', 't', '.', 'c', 'h', 'a', 'n', 'g', 'e', 'd', '\0',
  /*  9314 - "searchEngineDefault.displayName" */ 's', 'e', 'a', 'r', 'c', 'h', 'E', 'n', 'g', 'i', 'n', 'e', 'D', 'e', 'f', 'a', 'u', 'l', 't', '.', 'd', 'i', 's', 'p', 'l', 'a', 'y', 'N', 'a', 'm', 'e', '\0',
  /*  9346 - "searchEngineDefault.engineId" */ 's', 'e', 'a', 'r', 'c', 'h', 'E', 'n', 'g', 'i', 'n', 'e', 'D', 'e', 'f', 'a', 'u', 'l', 't', '.', 'e', 'n', 'g', 'i', 'n', 'e', 'I', 'd', '\0',
  /*  9375 - "searchEngineDefault.loadPath" */ 's', 'e', 'a', 'r', 'c', 'h', 'E', 'n', 'g', 'i', 'n', 'e', 'D', 'e', 'f', 'a', 'u', 'l', 't', '.', 'l', 'o', 'a', 'd', 'P', 'a', 't', 'h', '\0',
  /*  9404 - "searchEngineDefault.submissionUrl" */ 's', 'e', 'a', 'r', 'c', 'h', 'E', 'n', 'g', 'i', 'n', 'e', 'D', 'e', 'f', 'a', 'u', 'l', 't', '.', 's', 'u', 'b', 'm', 'i', 's', 's', 'i', 'o', 'n', 'U', 'r', 'l', '\0',
  /*  9438 - "searchEngineDefault.verified" */ 's', 'e', 'a', 'r', 'c', 'h', 'E', 'n', 'g', 'i', 'n', 'e', 'D', 'e', 'f', 'a', 'u', 'l', 't', '.', 'v', 'e', 'r', 'i', 'f', 'i', 'e', 'd', '\0',
  /*  9467 - "searchEnginePrivate.changed" */ 's', 'e', 'a', 'r', 'c', 'h', 'E', 'n', 'g', 'i', 'n', 'e', 'P', 'r', 'i', 'v', 'a', 't', 'e', '.', 'c', 'h', 'a', 'n', 'g', 'e', 'd', '\0',
  /*  9495 - "searchEnginePrivate.displayName" */ 's', 'e', 'a', 'r', 'c', 'h', 'E', 'n', 'g', 'i', 'n', 'e', 'P', 'r', 'i', 'v', 'a', 't', 'e', '.', 'd', 'i', 's', 'p', 'l', 'a', 'y', 'N', 'a', 'm', 'e', '\0',
  /*  9527 - "searchEnginePrivate.engineId" */ 's', 'e', 'a', 'r', 'c', 'h', 'E', 'n', 'g', 'i', 'n', 'e', 'P', 'r', 'i', 'v', 'a', 't', 'e', '.', 'e', 'n', 'g', 'i', 'n', 'e', 'I', 'd', '\0',
  /*  9556 - "searchEnginePrivate.loadPath" */ 's', 'e', 'a', 'r', 'c', 'h', 'E', 'n', 'g', 'i', 'n', 'e', 'P', 'r', 'i', 'v', 'a', 't', 'e', '.', 'l', 'o', 'a', 'd', 'P', 'a', 't', 'h', '\0',
  /*  9585 - "searchEnginePrivate.submissionUrl" */ 's', 'e', 'a', 'r', 'c', 'h', 'E', 'n', 'g', 'i', 'n', 'e', 'P', 'r', 'i', 'v', 'a', 't', 'e', '.', 's', 'u', 'b', 'm', 'i', 's', 's', 'i', 'o', 'n', 'U', 'r', 'l', '\0',
  /*  9619 - "searchEnginePrivate.verified" */ 's', 'e', 'a', 'r', 'c', 'h', 'E', 'n', 'g', 'i', 'n', 'e', 'P', 'r', 'i', 'v', 'a', 't', 'e', '.', 'v', 'e', 'r', 'i', 'f', 'i', 'e', 'd', '\0',
  /*  9648 - "searchService.startupTime" */ 's', 'e', 'a', 'r', 'c', 'h', 'S', 'e', 'r', 'v', 'i', 'c', 'e', '.', 's', 't', 'a', 'r', 't', 'u', 'p', 'T', 'i', 'm', 'e', '\0',
  /*  9674 - "dap.reportGenerationStatus" */ 'd', 'a', 'p', '.', 'r', 'e', 'p', 'o', 'r', 't', 'G', 'e', 'n', 'e', 'r', 'a', 't', 'i', 'o', 'n', 'S', 't', 'a', 't', 'u', 's', '\0',
  /*  9701 - "dap.uploadStatus" */ 'd', 'a', 'p', '.', 'u', 'p', 'l', 'o', 'a', 'd', 'S', 't', 'a', 't', 'u', 's', '\0',
  /*  9718 - "legacyTelemetry.clientId" */ 'l', 'e', 'g', 'a', 'c', 'y', 'T', 'e', 'l', 'e', 'm', 'e', 't', 'r', 'y', '.', 'c', 'l', 'i', 'e', 'n', 't', 'I', 'd', '\0',
  /*  9743 - "translations.error" */ 't', 'r', 'a', 'n', 's', 'l', 'a', 't', 'i', 'o', 'n', 's', '.', 'e', 'r', 'r', 'o', 'r', '\0',
  /*  9762 - "translations.errorRate" */ 't', 'r', 'a', 'n', 's', 'l', 'a', 't', 'i', 'o', 'n', 's', '.', 'e', 'r', 'r', 'o', 'r', 'R', 'a', 't', 'e', '\0',
  /*  9785 - "translations.requestsCount" */ 't', 'r', 'a', 'n', 's', 'l', 'a', 't', 'i', 'o', 'n', 's', '.', 'r', 'e', 'q', 'u', 'e', 's', 't', 's', 'C', 'o', 'u', 'n', 't', '\0',
  /*  9812 - "translations.restorePage" */ 't', 'r', 'a', 'n', 's', 'l', 'a', 't', 'i', 'o', 'n', 's', '.', 'r', 'e', 's', 't', 'o', 'r', 'e', 'P', 'a', 'g', 'e', '\0',
  /*  9837 - "translations.translationRequest" */ 't', 'r', 'a', 'n', 's', 'l', 'a', 't', 'i', 'o', 'n', 's', '.', 't', 'r', 'a', 'n', 's', 'l', 'a', 't', 'i', 'o', 'n', 'R', 'e', 'q', 'u', 'e', 's', 't', '\0',
  /*  9869 - "translationsPanel.aboutTranslations" */ 't', 'r', 'a', 'n', 's', 'l', 'a', 't', 'i', 'o', 'n', 's', 'P', 'a', 'n', 'e', 'l', '.', 'a', 'b', 'o', 'u', 't', 'T', 'r', 'a', 'n', 's', 'l', 'a', 't', 'i', 'o', 'n', 's', '\0',
  /*  9905 - "translationsPanel.alwaysOfferTranslations" */ 't', 'r', 'a', 'n', 's', 'l', 'a', 't', 'i', 'o', 'n', 's', 'P', 'a', 'n', 'e', 'l', '.', 'a', 'l', 'w', 'a', 'y', 's', 'O', 'f', 'f', 'e', 'r', 'T', 'r', 'a', 'n', 's', 'l', 'a', 't', 'i', 'o', 'n', 's', '\0',
  /*  9947 - "translationsPanel.alwaysTranslateLanguage" */ 't', 'r', 'a', 'n', 's', 'l', 'a', 't', 'i', 'o', 'n', 's', 'P', 'a', 'n', 'e', 'l', '.', 'a', 'l', 'w', 'a', 'y', 's', 'T', 'r', 'a', 'n', 's', 'l', 'a', 't', 'e', 'L', 'a', 'n', 'g', 'u', 'a', 'g', 'e', '\0',
  /*  9989 - "translationsPanel.cancelButton" */ 't', 'r', 'a', 'n', 's', 'l', 'a', 't', 'i', 'o', 'n', 's', 'P', 'a', 'n', 'e', 'l', '.', 'c', 'a', 'n', 'c', 'e', 'l', 'B', 'u', 't', 't', 'o', 'n', '\0',
  /* 10020 - "translationsPanel.changeFromLanguage" */ 't', 'r', 'a', 'n', 's', 'l', 'a', 't', 'i', 'o', 'n', 's', 'P', 'a', 'n', 'e', 'l', '.', 'c', 'h', 'a', 'n', 'g', 'e', 'F', 'r', 'o', 'm', 'L', 'a', 'n', 'g', 'u', 'a', 'g', 'e', '\0',
  /* 10057 - "translationsPanel.changeSourceLanguageButton" */ 't', 'r', 'a', 'n', 's', 'l', 'a', 't', 'i', 'o', 'n', 's', 'P', 'a', 'n', 'e', 'l', '.', 'c', 'h', 'a', 'n', 'g', 'e', 'S', 'o', 'u', 'r', 'c', 'e', 'L', 'a', 'n', 'g', 'u', 'a', 'g', 'e', 'B', 'u', 't', 't', 'o', 'n', '\0',
  /* 10102 - "translationsPanel.changeToLanguage" */ 't', 'r', 'a', 'n', 's', 'l', 'a', 't', 'i', 'o', 'n', 's', 'P', 'a', 'n', 'e', 'l', '.', 'c', 'h', 'a', 'n', 'g', 'e', 'T', 'o', 'L', 'a', 'n', 'g', 'u', 'a', 'g', 'e', '\0',
  /* 10137 - "translationsPanel.close" */ 't', 'r', 'a', 'n', 's', 'l', 'a', 't', 'i', 'o', 'n', 's', 'P', 'a', 'n', 'e', 'l', '.', 'c', 'l', 'o', 's', 'e', '\0',
  /* 10161 - "translationsPanel.closeFromLanguageMenu" */ 't', 'r', 'a', 'n', 's', 'l', 'a', 't', 'i', 'o', 'n', 's', 'P', 'a', 'n', 'e', 'l', '.', 'c', 'l', 'o', 's', 'e', 'F', 'r', 'o', 'm', 'L', 'a', 'n', 'g', 'u', 'a', 'g', 'e', 'M', 'e', 'n', 'u', '\0',
  /* 10201 - "translationsPanel.closeSettingsMenu" */ 't', 'r', 'a', 'n', 's', 'l', 'a', 't', 'i', 'o', 'n', 's', 'P', 'a', 'n', 'e', 'l', '.', 'c', 'l', 'o', 's', 'e', 'S', 'e', 't', 't', 'i', 'n', 'g', 's', 'M', 'e', 'n', 'u', '\0',
  /* 10237 - "translationsPanel.closeToLanguageMenu" */ 't', 'r', 'a', 'n', 's', 'l', 'a', 't', 'i', 'o', 'n', 's', 'P', 'a', 'n', 'e', 'l', '.', 'c', 'l', 'o', 's', 'e', 'T', 'o', 'L', 'a', 'n', 'g', 'u', 'a', 'g', 'e', 'M', 'e', 'n', 'u', '\0',
  /* 10275 - "translationsPanel.dismissErrorButton" */ 't', 'r', 'a', 'n', 's', 'l', 'a', 't', 'i', 'o', 'n', 's', 'P', 'a', 'n', 'e', 'l', '.', 'd', 'i', 's', 'm', 'i', 's', 's', 'E', 'r', 'r', 'o', 'r', 'B', 'u', 't', 't', 'o', 'n', '\0',
  /* 10312 - "translationsPanel.learnMore" */ 't', 'r', 'a', 'n', 's', 'l', 'a', 't', 'i', 'o', 'n', 's', 'P', 'a', 'n', 'e', 'l', '.', 'l', 'e', 'a', 'r', 'n', 'M', 'o', 'r', 'e', '\0',
  /* 10340 - "translationsPanel.manageLanguages" */ 't', 'r', 'a', 'n', 's', 'l', 'a', 't', 'i', 'o', 'n', 's', 'P', 'a', 'n', 'e', 'l', '.', 'm', 'a', 'n', 'a', 'g', 'e', 'L', 'a', 'n', 'g', 'u', 'a', 'g', 'e', 's', '\0',
  /* 10374 - "translationsPanel.neverTranslateLanguage" */ 't', 'r', 'a', 'n', 's', 'l', 'a', 't', 'i', 'o', 'n', 's', 'P', 'a', 'n', 'e', 'l', '.', 'n', 'e', 'v', 'e', 'r', 'T', 'r', 'a', 'n', 's', 'l', 'a', 't', 'e', 'L', 'a', 'n', 'g', 'u', 'a', 'g', 'e', '\0',
  /* 10415 - "translationsPanel.neverTranslateSite" */ 't', 'r', 'a', 'n', 's', 'l', 'a', 't', 'i', 'o', 'n', 's', 'P', 'a', 'n', 'e', 'l', '.', 'n', 'e', 'v', 'e', 'r', 'T', 'r', 'a', 'n', 's', 'l', 'a', 't', 'e', 'S', 'i', 't', 'e', '\0',
  /* 10452 - "translationsPanel.open" */ 't', 'r', 'a', 'n', 's', 'l', 'a', 't', 'i', 'o', 'n', 's', 'P', 'a', 'n', 'e', 'l', '.', 'o', 'p', 'e', 'n', '\0',
  /* 10475 - "translationsPanel.openFromLanguageMenu" */ 't', 'r', 'a', 'n', 's', 'l', 'a', 't', 'i', 'o', 'n', 's', 'P', 'a', 'n', 'e', 'l', '.', 'o', 'p', 'e', 'n', 'F', 'r', 'o', 'm', 'L', 'a', 'n', 'g', 'u', 'a', 'g', 'e', 'M', 'e', 'n', 'u', '\0',
  /* 10514 - "translationsPanel.openSettingsMenu" */ 't', 'r', 'a', 'n', 's', 'l', 'a', 't', 'i', 'o', 'n', 's', 'P', 'a', 'n', 'e', 'l', '.', 'o', 'p', 'e', 'n', 'S', 'e', 't', 't', 'i', 'n', 'g', 's', 'M', 'e', 'n', 'u', '\0',
  /* 10549 - "translationsPanel.openToLanguageMenu" */ 't', 'r', 'a', 'n', 's', 'l', 'a', 't', 'i', 'o', 'n', 's', 'P', 'a', 'n', 'e', 'l', '.', 'o', 'p', 'e', 'n', 'T', 'o', 'L', 'a', 'n', 'g', 'u', 'a', 'g', 'e', 'M', 'e', 'n', 'u', '\0',
  /* 10586 - "translationsPanel.restorePageButton" */ 't', 'r', 'a', 'n', 's', 'l', 'a', 't', 'i', 'o', 'n', 's', 'P', 'a', 'n', 'e', 'l', '.', 'r', 'e', 's', 't', 'o', 'r', 'e', 'P', 'a', 'g', 'e', 'B', 'u', 't', 't', 'o', 'n', '\0',
  /* 10622 - "translationsPanel.translateButton" */ 't', 'r', 'a', 'n', 's', 'l', 'a', 't', 'i', 'o', 'n', 's', 'P', 'a', 'n', 'e', 'l', '.', 't', 'r', 'a', 'n', 's', 'l', 'a', 't', 'e', 'B', 'u', 't', 't', 'o', 'n', '\0',
  /* 10656 - "firstStartup.deleteTasksTime" */ 'f', 'i', 'r', 's', 't', 'S', 't', 'a', 'r', 't', 'u', 'p', '.', 'd', 'e', 'l', 'e', 't', 'e', 'T', 'a', 's', 'k', 's', 'T', 'i', 'm', 'e', '\0',
  /* 10685 - "firstStartup.elapsed" */ 'f', 'i', 'r', 's', 't', 'S', 't', 'a', 'r', 't', 'u', 'p', '.', 'e', 'l', 'a', 'p', 's', 'e', 'd', '\0',
  /* 10706 - "firstStartup.normandyInitTime" */ 'f', 'i', 'r', 's', 't', 'S', 't', 'a', 'r', 't', 'u', 'p', '.', 'n', 'o', 'r', 'm', 'a', 'n', 'd', 'y', 'I', 'n', 'i', 't', 'T', 'i', 'm', 'e', '\0',
  /* 10736 - "firstStartup.statusCode" */ 'f', 'i', 'r', 's', 't', 'S', 't', 'a', 'r', 't', 'u', 'p', '.', 's', 't', 'a', 't', 'u', 's', 'C', 'o', 'd', 'e', '\0',
  /* 10760 - "blocklist.addonBlockChange" */ 'b', 'l', 'o', 'c', 'k', 'l', 'i', 's', 't', '.', 'a', 'd', 'd', 'o', 'n', 'B', 'l', 'o', 'c', 'k', 'C', 'h', 'a', 'n', 'g', 'e', '\0',
  /* 10787 - "blocklist.lastModifiedRsAddonsMblf" */ 'b', 'l', 'o', 'c', 'k', 'l', 'i', 's', 't', '.', 'l', 'a', 's', 't', 'M', 'o', 'd', 'i', 'f', 'i', 'e', 'd', 'R', 's', 'A', 'd', 'd', 'o', 'n', 's', 'M', 'b', 'l', 'f', '\0',
  /* 10822 - "blocklist.mlbfGenerationTime" */ 'b', 'l', 'o', 'c', 'k', 'l', 'i', 's', 't', '.', 'm', 'l', 'b', 'f', 'G', 'e', 'n', 'e', 'r', 'a', 't', 'i', 'o', 'n', 'T', 'i', 'm', 'e', '\0',
  /* 10851 - "blocklist.mlbfSource" */ 'b', 'l', 'o', 'c', 'k', 'l', 'i', 's', 't', '.', 'm', 'l', 'b', 'f', 'S', 'o', 'u', 'r', 'c', 'e', '\0',
  /* 10872 - "blocklist.mlbfStashTimeNewest" */ 'b', 'l', 'o', 'c', 'k', 'l', 'i', 's', 't', '.', 'm', 'l', 'b', 'f', 'S', 't', 'a', 's', 'h', 'T', 'i', 'm', 'e', 'N', 'e', 'w', 'e', 's', 't', '\0',
  /* 10902 - "blocklist.mlbfStashTimeOldest" */ 'b', 'l', 'o', 'c', 'k', 'l', 'i', 's', 't', '.', 'm', 'l', 'b', 'f', 'S', 't', 'a', 's', 'h', 'T', 'i', 'm', 'e', 'O', 'l', 'd', 'e', 's', 't', '\0',
  /* 10932 - "update.autoDownload" */ 'u', 'p', 'd', 'a', 't', 'e', '.', 'a', 'u', 't', 'o', 'D', 'o', 'w', 'n', 'l', 'o', 'a', 'd', '\0',
  /* 10952 - "update.backgroundUpdate" */ 'u', 'p', 'd', 'a', 't', 'e', '.', 'b', 'a', 'c', 'k', 'g', 'r', 'o', 'u', 'n', 'd', 'U', 'p', 'd', 'a', 't', 'e', '\0',
  /* 10976 - "update.canUsuallyApplyUpdates" */ 'u', 'p', 'd', 'a', 't', 'e', '.', 'c', 'a', 'n', 'U', 's', 'u', 'a', 'l', 'l', 'y', 'A', 'p', 'p', 'l', 'y', 'U', 'p', 'd', 'a', 't', 'e', 's', '\0',
  /* 11006 - "update.canUsuallyCheckForUpdates" */ 'u', 'p', 'd', 'a', 't', 'e', '.', 'c', 'a', 'n', 'U', 's', 'u', 'a', 'l', 'l', 'y', 'C', 'h', 'e', 'c', 'k', 'F', 'o', 'r', 'U', 'p', 'd', 'a', 't', 'e', 's', '\0',
  /* 11039 - "update.canUsuallyStageUpdates" */ 'u', 'p', 'd', 'a', 't', 'e', '.', 'c', 'a', 'n', 'U', 's', 'u', 'a', 'l', 'l', 'y', 'S', 't', 'a', 'g', 'e', 'U', 'p', 'd', 'a', 't', 'e', 's', '\0',
  /* 11069 - "update.canUsuallyUseBits" */ 'u', 'p', 'd', 'a', 't', 'e', '.', 'c', 'a', 'n', 'U', 's', 'u', 'a', 'l', 'l', 'y', 'U', 's', 'e', 'B', 'i', 't', 's', '\0',
  /* 11094 - "update.channel" */ 'u', 'p', 'd', 'a', 't', 'e', '.', 'c', 'h', 'a', 'n', 'n', 'e', 'l', '\0',
  /* 11109 - "update.enabled" */ 'u', 'p', 'd', 'a', 't', 'e', '.', 'e', 'n', 'a', 'b', 'l', 'e', 'd', '\0',
  /* 11124 - "update.serviceEnabled" */ 'u', 'p', 'd', 'a', 't', 'e', '.', 's', 'e', 'r', 'v', 'i', 'c', 'e', 'E', 'n', 'a', 'b', 'l', 'e', 'd', '\0',
  /* 11146 - "timerThread.timersFiredPerWakeup" */ 't', 'i', 'm', 'e', 'r', 'T', 'h', 'r', 'e', 'a', 'd', '.', 't', 'i', 'm', 'e', 'r', 's', 'F', 'i', 'r', 'e', 'd', 'P', 'e', 'r', 'W', 'a', 'k', 'e', 'u', 'p', '\0',
};


static_assert(sizeof(gMetricStringTable) < 4294967296, "Metric string table is too large.");

const metric_entry_t sMetricByNameLookupEntries[] = {
  8070451137838321615ull,
  5764608618250902918ull,
  2305843408645654995ull,
  4035225652671023456ull,
  2882303997740320446ull,
  5188147818702838896ull,
  2882305174561367351ull,
  2882303894661104598ull,
  5764608961848288741ull,
  2305843816667551036ull,
  1152923106629659286ull,
  1729382351399551705ull,
  4611686173046211697ull,
  10376294563663846359ull,
  5764608729920053221ull,
  6917529671886180507ull,
  5764608068495084890ull,
  4035226524549390620ull,
  3458765480188189211ull,
  8646912435602595522ull,
  4035226846671940104ull,
  2882303929020843193ull,
  3458764565360148855ull,
  2305843782307812348ull,
  1729382557557983191ull,
  6917529659001278560ull,
  2882305354949995107ull,
  5764608459337111821ull,
  2305843030688530570ull,
  2305843795192714357ull,
  5764608077085019574ull,
  1729382269795172429ull,
  5764608790049595731ull,
  8070451107773550235ull,
  5764608863064040184ull,
  2882303924725875872ull,
  4035225515232069370ull,
  2305844615531473588ull,
  3458765304094528663ull,
  2305843507429903383ull,
  9799833905849703951ull,
  1729382677817068137ull,
  8646912001810895462ull,
  8646911993220960800ull,
  6341068726309227289ull,
  4035225734275402609ull,
  4035226670578279675ull,
  4035226756477626185ull,
  576460795253096758ull,
  2882304216783653683ull,
  2305843296976504722ull,
  6917529985418795473ull,
  2305843288386570100ull,
  2882305153086530719ull,
  4035226266851351381ull,
  5188146942529504464ull,
  8070451120658452246ull,
  6917529749195592329ull,
  5764608867359007511ull,
  5188147273241988146ull,
  2305844632711342879ull,
  4035225498052200112ull,
  9223373140661378497ull,
  4035226477304750087ull,
  3458765316979430675ull,
  2305843417235589652ull,
  2305843352811079814ull,
  3458766077188647328ull,
  9799833901554736635ull,
  4035226760772593526ull,
  2882305178856334676ull,
  7493990312520453378ull,
  4035225876009324544ull,
  5764608072790052231ull,
  10376294610908486875ull,
  2305843494545001403ull,
  6917529663296245879ull,
  4035225644081088832ull,
  2882303851711431356ull,
  4035225622606252242ull,
  8070451983946884642ull,
  2305844134495133256ull,
  2882303920430908555ull,
  5188147810112904235ull,
  1152922659953057498ull,
  4035226520254423298ull,
  8070451133543354273ull,
  4035226309801024591ull,
  6341069696971842929ull,
  3458765488778123881ull,
  2305843786602779679ull,
  6341068588870273038ull,
  5764608081379986917ull,
  5764607669063124024ull,
  5764608270358549305ull,
  4035226026333180814ull,
  4035226253966449376ull,
  4035225532411938630ull,
  5764608893128811503ull,
  2882304002035287766ull,
  2882304066459797482ull,
  2882304079344699428ull,
  8646911963156189509ull,
  2305844014236048251ull,
  5764608433567307861ull,
  2305843073638203887ull,
  2882303847416464037ull,
  5764608605366000941ull,
  4035226060692919487ull,
  5764608047020248261ull,
  2305843503134936059ull,
  2882304186718882432ull,
  6917530402030625812ull,
  2882305161676465374ull,
  4035226511664488656ull,
  2882304723589798389ull,
  9223372900143208261ull,
  4035226730707822189ull,
  4035225661260958119ull,
  6917530457865201072ull,
  5188147999091466357ull,
  8070451099183615555ull,
  2882303946200712496ull,
  4035226825197103486ull,
  4035225541001873250ull,
  2882305393604701014ull,
  576460782368194759ull,
  3458765020626685008ull,
  6917529912404350945ull,
  3458765682051653408ull,
  2305844061480688769ull,
  2305844645596244837ull,
  2305843438710426265ull,
  8646911997515928131ull,
  2305843889681995713ull,
  8070451124953419591ull,
  3458765531727797185ull,
  3458766090073549296ull,
  6917529650411343931ull,
  2882303950495679827ull,
  576461817455320257ull,
  5764608880243909507ull,
  5764607699127895280ull,
  4035225631196186881ull,
  5188147436450746625ull,
  2882304006330255076ull,
  1152923089449789987ull,
  2882303941905745164ull,
  3458765312684463336ull,
  4035226262556384046ull,
  1729383390781644403ull,
  6917529684771082456ull,
  576460979936691865ull,
  3458765269734789995ull,
  6917529676181147821ull,
  2305844624121408224ull,
  2882303787286921387ull,
  6917529959648991465ull,
  4035226743592724187ull,
  2882303873186267974ull,
  4035226782247430179ull,
  8646912564451615103ull,
  4035226816607168821ull,
  2305844637006310205ull,
  5764608399207569314ull,
  8070451103478582893ull,
  4035226258261416707ull,
  1152921513196781604ull,
  1152923093744757318ull,
  6917529981123828137ull,
  5764608274653516627ull,
  6341069671202038972ull,
  6917529689066049772ull,
  5188148072105910768ull,
  5188148084990812732ull,
  4611687199543402338ull,
  5764608888833844172ull,
  3458764960497142531ull,
  4035225339138409010ull,
  7493990316815420693ull,
  5764608871653974840ull,
  6917529727720755686ull,
  4035226795132332166ull,
  5764608412092471299ull,
  2882303890366137274ull,
  3458765364224071375ull,
  8646911967451156838ull,
  4035225738570369923ull,
  2882304199603784406ull,
  10952755384686746988ull,
  1729383450911186845ull,
  5764608253178680014ull,
  5188148059221008795ull,
  3458766081483614653ull,
  5764608266063581977ull,
  5188147445040681266ull,
  10952755367506877682ull,
  10376294482059467059ull,
  2305844151675002523ull,
  3458765329864332691ull,
  576460829612835421ull,
  6917529732015723008ull,
  2882303911840973914ull,
  4035225549591807878ull,
  4035225648376056144ull,
  5764608798639530351ull,
  5188148080695845415ull,
  4035225742865337235ull,
  2305844628416375550ull,
  5764608609660968269ull,
  5764608257473647347ull,
  5764608051315215593ull,
  5764608845884170858ull,
  3458765441533483201ull,
  2882304934043197238ull,
  2882304083639666740ull,
  2305843498839968732ull,
  2882304732179733057ull,
  4035226747887691525ull,
  8646912611696255622ull,
  6341069465043607436ull,
  7493991235938428473ull,
  5188148089285780049ull,
  2882303959085614490ull,
  4035226485894684724ull,
  2305843851027289728ull,
  5188147801522969600ull,
  10376294490649401729ull,
  6917529719130821038ull,
  2305843855322257060ull,
  4035226095052658191ull,
  4035226249671482043ull,
  4035226820902136154ull,
  2305843069343236555ull,
  9223372895848240927ull,
  4035226773657495513ull,
  6917530213052063604ull,
  8070451756313616461ull,
  3458764582540018192ull,
  9799833695396304968ull,
  5764608042725280930ull,
  5764608820114366930ull,
  2305843820962518372ull,
  5188147805817936917ull,
  4035226507369521336ull,
  4035225592541481028ull,
  6917530165807423109ull,
  1152922406549985341ull,
  5188148054926041488ull,
  4035226799427299503ull,
  2882304221078621002ull,
  5764608815819399604ull,
  2305843829552453060ull,
  1729383700019291638ull,
  6917529667591213191ull,
  4035226812312201490ull,
  4035225558181742513ull,
  6917529401303238948ull,
  2882305187446269331ull,
  3458765295504593993ull,
  5764608193049137441ull,
  2882303971970516493ull,
  2882303860301366010ull,
  3458765282619691992ull,
  1152923102334691960ull,
  2882304057869862851ull,
  1152921551851487566ull,
  8070451116363484909ull,
  4035225601131415652ull,
  3458765347044202036ull,
  4035225626901219559ull,
  3458764823058188284ull,
  4035226133707364204ull,
  5764608210229006729ull,
  9223373157841247784ull,
  576460833907802752ull,
  4035226765067560857ull,
  5188146908169765874ull,
  2305843412940622326ull,
  4035226752182658852ull,
  6917529680476115134ull,
  2882303976265483823ull,
  3458764853122959480ull,
  4035226245376514711ull,
  576460756598390784ull,
  4035225618311284923ull,
  4035225635491154194ull,
  8070451112068517577ull,
  8070451073413811515ull,
  10952755371801845007ull,
  4035226644808475718ull,
  5764608682675412716ull,
  6917529654706311242ull,
  4611686121506603795ull,
  4035225656965990785ull,
  3458765338454267371ull,
  8070451094888648215ull,
  9799833703986239597ull,
  5764608858769072850ull,
  5764607742077568589ull,
  3458765278324724658ull,
  2882305148791563394ull,
  6917529641821409294ull,
  7493990982535356355ull,
  1729382364284453677ull,
  2305843339926177875ull,
  5764608957553321418ull,
  5764608794344563041ull,
  2882305170266400023ull,
  5188146890989896569ull,
  1152922634183253600ull,
  576461005706495752ull,
  6917529972533893466ull,
  5188147917487087277ull,
  4035226515959455974ull,
  4035226769362528177ull,
  2882303967675549154ull,
  4035225614016317600ull,
  2305843266911733533ull,
  5188147281831922799ull,
  5764608785754628410ull,
  1729382398644192283ull,
  4035226503074554014ull,
  6917530054138272746ull,
  6917529753490559660ull,
  3458765621922110948ull,
  5764608884538876843ull,
  2882303954790647159ull,
  4611686134391505756ull,
  2305843047868399884ull,
  9799833845720161430ull,
  4035226790837364836ull,
  2305843279796635475ull,
  4035225528116971314ull,
  2882304178128947796ull,
  1152921560441422243ull,
  4035226739297756849ull,
  5188147784343100329ull,
  2882303984855418484ull,
  2305843443005393599ull,
  4035226713527952911ull,
  2305843026393563239ull,
  4035226735002789517ull,
  2305843043573432544ull,
  4035225729980435294ull,
  576461929124470602ull,
  5764608261768614662ull,
  2305843906861865007ull,
  2882304822374046891ull,
  8070451129248386931ull,
  2882304654870321181ull,
  2882303886071169952ull,
  4035226808017234155ull,
  2305844649891212148ull,
  2305843928336701581ull,
  3458764994856881061ull,
  2882305144496596066ull,
  3458765351339169371ull,
  5764608875948942171ull,
  9223373136366411173ull,
  3458766085778581970ull,
  6917529723425788365ull,
  5764608828704301577ull,
  9799833798475520918ull,
  4035226726412854868ull,
  5764608394912601985ull,
  4035226464419848111ull,
  2305844619826440904ull,
  2882303963380581824ull,
  2305843863912191740ull,
  5764608854474105516ull,
  4035226786542397512ull,
  4035226498779586698ull,
  2882304173833980476ull,
  6917529646116376617ull,
  8646912929523837834ull,
  5188147286126890122ull,
  11529216261544222754ull,
  2882304203898751725ull,
  2305843932631668904ull,
  4035226803722266836ull,
  10376294705397767944ull,
  4035226777952462845ull
};



Maybe<uint32_t>
MetricByNameLookup(const nsACString& aKey)
{
  static const uint16_t BASES[] = {
       1,   0,   3,   0,   4,   2,   0,   0,   1,   1,   0,   2,   1,   1,   0,   0,
       1,   0,   3,   0,   0,   0,   2,   1,   1,   6,   0,   4,   5,   3,   1,   0,
       1,   2,   3,   1,   1,   1,   0,   1,   0,   2,   1,   2,   0,   0,   0,   0,
       0,   0,   0,   0,   1,   0,   0,   2,   0,   0,   5,   0,   0,   0,   2,   9,
       1,   2,   7,   0,   2,   0,   2,   0,   0,   1,   7,   0,   0,   0,   1,   0,
       0,   1,   3,   2,   1,   1,   0,   0,   0,   1,   0,   0,   1,   0,   0,   0,
       1,  15,   0,   1,   0,   0,   0,   0,   0,   0,   0,   0,   6,   0,   2,   0,
       0,   5,   0,   1,   1,   0,   1,   1,   0,   1,   0,   0,   0,   1,   1,   0,
       0,   5,   0,   2,   1,   2,   0,   0,   1,   3,   0,   0,   4,   0,   4,   2,
       2,   4,   0,   4,   6,   0,   0,   3,   0,   0,   0,   0,   0,   3,   1,   0,
       0,   0,   0,   0,   0,   1,   0,   1,   0,   0,   4,   1,   1,   0,   1,   7,
       0,   1,   0,   1,   1,   0,   0,   0,   7,   0,   1,   4,   0,   3,   7,   2,
       3,   0,   0,   0,   0,   0,   2,   1,   0,   1,   0,   0,   1,   0,   0,   1,
      12,   0,   0,   0,   0,   0,   8,   3,   0,   3,   0,   1,   6,   4,   2,   0,
       0,   0,   2,   3,   2,   3,   0,   6,   0,   0,   0,   1,   1,   0,   1,   9,
       5,   2,   1,   1,   0,   2,   1,   7,   6,   0,   0,   0,   0,   0,   3,   2,
      15,  13,   0,   1,   0,   0,   0,   1,   2,  11,   1,   2,   1,   1,   3,   5,
       4,   0,   0,   2,   1,   0,   0,   3,   1,   4,   0,   1,   0,   1,   0,   7,
      12,   1,   0,   0,   1,   0,   1,   0,   0,   0,   2,   0,   3,   1,   0,   3,
       4,   7,   0,   0,   0,   5,   0,   0,   0,   2,   4,   0,   0,   2,   0,   0,
       0,   0,   3,   4,   1,   0,   1,   0,   3,   4,   0,   4,   0,   0,   0,   0,
       0,   7,   7,  13,   0,   0,   0,   0,   2,   0,   0,   3,   2,   1,   1,   1,
       3,   1,   7,   0,   0,   8,   9,   5,   4,   0,  27,  19,   2,   0,   6,   0,
       4,  33,   0,   0,   1,   0,   1,  18,  14,  16,   0,  13,   8,   0,   0,   0,
       1,   4,  37,   1,   0,   7,   3,   0,   8,   0,  13,   0,  45,   3,   2,   0,
       1,  12,  14,  10,  17,   1,   1,   0,  52,   0,   0,   0,   1,  16,   0,   0,
       0,   0,   3,  12,  34,   0,   0,   1,   0,  19,   0,   2,   0,   0,   8,   3,
       0,   8,   0,   0,   0,   0,   0,   7,   0,   0,   0,   1,   0,   0,   0,   0,
       7,   1,  24,   0,   1,   7,   0,  29,   0,   0,  19,   7,   6,  24,   0,  11,
       0,   0,   9,   0,  29,   0,   1,   0,  13,   0,  20,   0,   0,   1,  14,   0,
       0,   0,  21,   0,  76,   0,   0,   0,  13,   3,  14,   0,   2,   1,   1,   0,
      38,   0,   1,  29,  20,   0, 151, 159,   0,  30,   0,   0,   0,   0, 401, 146,
  };
  

  const char* bytes = aKey.BeginReading();
  size_t length = aKey.Length();
  auto& entry = mozilla::perfecthash::Lookup(bytes, length, BASES,
                                             sMetricByNameLookupEntries);
  return metric_result_check(aKey, entry);
}


/**
 * Get a category's name from the string table.
 */
const char* GetCategoryName(category_entry_t entry) {
  MOZ_ASSERT(entry < sizeof(gCategoryStringTable), "Entry identifier offset larger than string table");
  return &gCategoryStringTable[entry];
}

/**
 * Get a metric's identifier from the string table.
 */
const char* GetMetricIdentifier(metric_entry_t entry) {
  uint32_t offset = GLEAN_OFFSET(entry);
  MOZ_ASSERT(offset < sizeof(gMetricStringTable), "Entry identifier offset larger than string table");
  return &gMetricStringTable[offset];
}

/**
 * Check that the found entry is pointing to the right key
 * and return it.
 * Or return `Nothing()` if the entry was not found.
 */
static Maybe<uint32_t> category_result_check(const nsACString& aKey, category_entry_t entry) {
  if (MOZ_UNLIKELY(entry > sizeof(gCategoryStringTable))) {
    return Nothing();
  }
  if (aKey.EqualsASCII(gCategoryStringTable + entry)) {
    return Some(entry);
  }
  return Nothing();
}

/**
 * Check if the found entry index is pointing to the right key
 * and return the corresponding metric ID.
 * Or return `Nothing()` if the entry was not found.
 */
static Maybe<uint32_t> metric_result_check(const nsACString& aKey, uint64_t entry) {
  uint32_t metricId = entry >> GLEAN_INDEX_BITS;
  uint32_t offset = GLEAN_OFFSET(entry);

  if (offset > sizeof(gMetricStringTable)) {
    return Nothing();
  }

  if (aKey.EqualsASCII(gMetricStringTable + offset)) {
    return Some(metricId);
  }

  return Nothing();
}


#undef GLEAN_INDEX_BITS
#undef GLEAN_ID_BITS
#undef GLEAN_TYPE_ID
#undef GLEAN_METRIC_ID
#undef GLEAN_OFFSET

} // namespace mozilla::glean
